include "RiversLakes"
include "MapUtilities"

--Original Author: Gedemon
--Changes made by totalslacker for the purposes of this mod

function Round(num)
    under = math.floor(num)
    upper = math.floor(num) + 1
    underV = -(under - num)
    upperV = upper - num
    if (upperV > underV) then
        return under
    else
        return upper
    end
end

local iTurn 				= Game.GetCurrentGameTurn()

local mapName = MapConfiguration.GetValue("MapName")
print("Map Name = " .. tostring(mapName))

local cityStateCount = GameConfiguration.GetValue("CITY_STATE_COUNT")
print("City State Count = " .. tostring(cityStateCount))
local csCounter = 0
local prevCounter = 0

local bAddGoodies = MapConfiguration.GetValue("ExtraTribes")
print("Extra tribes: " .. tostring(bAddGoodies))
local iAddGoodies = 0
local bNoGoodies = GameConfiguration.GetValue("GAME_NO_GOODY_HUTS")

local bAutoCityNaming 			= MapConfiguration.GetValue("RenameCities")
print("Auto city naming: " .. tostring(bAutoCityNaming))
local bCanUseCivSpecificName 	= true

local bHorsesAmericas			= MapConfiguration.GetValue("Horses")
local bCliffsOfDover			= MapConfiguration.GetValue("Cliffs")
local bDeadSea					= MapConfiguration.GetValue("DeadSea")
local bCraterLake				= MapConfiguration.GetValue("CraterLake")
local bVikingDLC				= MapConfiguration.GetValue("VikingDLC")

local idMaori	= -1

local bUseRelativePlacement = MapConfiguration.GetValue("UseRelativePlacement")
local g_ReferenceMapWidth 	= MapConfiguration.GetValue("ReferenceMapWidth") or 84
local g_ReferenceMapHeight 	= MapConfiguration.GetValue("ReferenceMapHeight") or 54

local g_iW, g_iH 	= Map.GetGridSize()
local g_ReferenceWidthFactor  = g_ReferenceMapWidth / g_iW 
local g_ReferenceHeightFactor = g_ReferenceMapHeight / g_iH
local g_ReferenceWidthRatio   = g_iW / g_ReferenceMapWidth 
local g_ReferenceHeightRatio  = g_iH / g_ReferenceMapHeight

local g_ExtraRange = 0
if bUseRelativePlacement then
	g_ExtraRange = 10 --Round(10 * math.sqrt(g_iW * g_iH) / math.sqrt(g_ReferenceMapWidth * g_ReferenceMapHeight))
end

local isInGame = {}
-- Create list of Civilizations and leaders in game
for iPlayer = 0, PlayerManager.GetWasEverAliveCount() - 1 do
	local CivilizationTypeName = PlayerConfigurations[iPlayer]:GetCivilizationTypeName()
	local LeaderTypeName = PlayerConfigurations[iPlayer]:GetLeaderTypeName()
	
	if CivilizationTypeName == "CIVILIZATION_MAORI" then
		idMaori = iPlayer
	end
	
	if CivilizationTypeName then isInGame[CivilizationTypeName] = true end
	if LeaderTypeName 		then isInGame[LeaderTypeName] 		= true end
end

--Resource index
local cotton = GameInfo.Resources['RESOURCE_COTTON'].Index
local citrus = GameInfo.Resources['RESOURCE_CITRUS'].Index
local horses = GameInfo.Resources['RESOURCE_HORSES'].Index
local incense = GameInfo.Resources['RESOURCE_INCENSE'].Index	
local wheat = GameInfo.Resources['RESOURCE_WHEAT'].Index
local bananas = GameInfo.Resources['RESOURCE_BANANAS'].Index
local coffee = GameInfo.Resources['RESOURCE_COFFEE'].Index
local sugar = GameInfo.Resources['RESOURCE_SUGAR'].Index
local wine = GameInfo.Resources['RESOURCE_WINE'].Index
local sheep = GameInfo.Resources['RESOURCE_SHEEP'].Index
local cattle = GameInfo.Resources['RESOURCE_CATTLE'].Index
local marble = GameInfo.Resources['RESOURCE_MARBLE'].Index
local gold = GameInfo.Resources['RESOURCE_GOLD'].Index
local copper = GameInfo.Resources['RESOURCE_COPPER'].Index

--Features index
local cliffs = GameInfo.Features['FEATURE_CLIFFS_DOVER'].Index
local forest = GameInfo.Features['FEATURE_FOREST'].Index

--Terrains index
local plains = GameInfo.Terrains['TERRAIN_PLAINS'].Index
local desert = GameInfo.Terrains['TERRAIN_DESERT'].Index
local desertHills = GameInfo.Terrains['TERRAIN_DESERT_HILLS'].Index

--Flow Direction Index
local south = 		FlowDirectionTypes.FLOWDIRECTION_SOUTH
local southEast = 	FlowDirectionTypes.FLOWDIRECTION_SOUTHEAST
local southWest = 	FlowDirectionTypes.FLOWDIRECTION_SOUTHWEST
local north = 		FlowDirectionTypes.FLOWDIRECTION_NORTH
local northEast = 	FlowDirectionTypes.FLOWDIRECTION_NORTHEAST
local northWest = 	FlowDirectionTypes.FLOWDIRECTION_NORTHWEST

----------------------------------------------------------------------------------------
-- City renaming <<<<<
----------------------------------------------------------------------------------------
if bAutoCityNaming ~= 0 then
----------------------------------------------------------------------------------------
print("Activating Auto City Naming...")
local nameUsed = {}
local nameUsedOnContinent = {}
local nameUsedByCivilization = {}

function SetExistingCityNames(excludingCity)
	nameUsedOnContinent = {}
	nameUsedByCivilization = {}
	for _, player_ID in ipairs(PlayerManager.GetWasEverAliveMajorIDs()) do
		local player = Players[player_ID]
		local playerConfig = PlayerConfigurations[player_ID]
		local civilization = playerConfig:GetCivilizationTypeName()
		local playerCities = player:GetCities();
		if civilization ~= "CIVILIZATION_CREE" and civilization ~= "CIVILIZATION_AZTEC" and civilization ~= "CIVILIZATION_MAPUCHE" and civilization ~= "CIVILIZATION_MAORI" then
			for j, city in playerCities:Members() do
				if city ~= excludingCity then -- do not add the current name of the city we are renaming, in case it's valid at the city position.
					local plot = Map.GetPlot(city:GetX(), city:GetY())
					local continent = plot:GetContinentType()
					
					nameUsed[city:GetName()] = true
					nameUsed[Locale.Lookup(city:GetName())] = true
					
					if not nameUsedOnContinent[continent] then
						nameUsedOnContinent[continent] = {}
					end
					nameUsedOnContinent[continent][city:GetName()] = true
					nameUsedOnContinent[continent][Locale.Lookup(city:GetName())] = true
					
					if not nameUsedByCivilization[civilization] then
						nameUsedByCivilization[civilization] = {}
					end
					nameUsedByCivilization[civilization][city:GetName()] = true
					nameUsedByCivilization[civilization][Locale.Lookup(city:GetName())] = true
				end
			end		
		end
	end
end

function IsNameUsedByCivilization(name, civilization)
	return nameUsedByCivilization[civilization] and (nameUsedByCivilization[civilization][name] or nameUsedByCivilization[civilization][Locale.Lookup(name)])
end

function IsNameUsedOnContinent(name, x, y)
	local plot = Map.GetPlot(x, y)
	local continent = plot:GetContinentType()
	return nameUsedOnContinent[continent] and (nameUsedOnContinent[continent][name] or nameUsedOnContinent[continent][Locale.Lookup(name)])
end

function ChangeCityName( ownerPlayerID, cityID)
	local pCity = CityManager.GetCity(ownerPlayerID, cityID)	
	if pCity then
		SetExistingCityNames(pCity)
		local mapX = pCity:GetX()
		local mapY = pCity:GetY()
		local refMapX, refMapY = mapX, mapY
		if bUseRelativePlacement then
			-- Convert plot position to the corresponding position on the reference map
			refMapX 	= Round(g_ReferenceWidthFactor * mapX)
			refMapY 	= Round(g_ReferenceHeightFactor * mapY)
		end
		local CivilizationTypeName = PlayerConfigurations[ownerPlayerID]:GetCivilizationTypeName()
		if CivilizationTypeName ~= "CIVILIZATION_CREE" and CivilizationTypeName ~= "CIVILIZATION_AZTEC" and CivilizationTypeName ~= "CIVILIZATION_MAPUCHE" and CivilizationTypeName ~= "CIVILIZATION_INCA" and CivilizationTypeName ~= "MAORI" and CivilizationTypeName ~= "CIVILIZATION_RL_SHOSHONE" then		
			local startPos, endPos = string.find(CivilizationTypeName, "CIVILIZATION_")
			local sCivSuffix = string.sub(CivilizationTypeName, endPos)
			print("Trying to find name for city of ".. tostring(CivilizationTypeName) .." at "..tostring(mapX)..","..tostring(mapY))
			local possibleName = {}
			local maxRange = 1
			local bestDistance = 99
			local bestDefaultDistance = 99
			local bestName = nil
			local bestDefaultName = nil
			local cityPlot = Map.GetPlot(mapX, mapY)
				for row in GameInfo.CityMap() do
					if row.MapName == mapName  then
						local name = row.CityLocaleName
						local nameX = row.X
						local nameY = row.Y
						local nameMaxDistance = (row.Area or maxRange) + g_ExtraRange
						-- rough selection in a square first before really testing distance
						if (math.abs(refMapX - nameX) <= nameMaxDistance) and (math.abs(refMapY - nameY) <= nameMaxDistance) then	
							--print("- testing "..tostring(name).." at "..tostring(nameX)..","..tostring(nameY).." max distance is "..tostring(nameMaxDistance)..", best distance so far is "..tostring(bestDistance))
							
							local distance = Map.GetPlotDistance(refMapX, refMapY ,nameX, nameY)
							if distance <= nameMaxDistance and distance < bestDistance then
							
								local sCityNameForCiv = tostring(name) .. sCivSuffix
							
								if bCanUseCivSpecificName and CivilizationTypeName == row.Civilization and not IsNameUsedByCivilization(name, CivilizationTypeName) then -- this city is specific to this Civilization, and the name is not already used
									bestDistance = distance
									bestName = name
								elseif not row.Civilization then -- do not use Civilization specific name with another Civilization, only generic							
									if bCanUseCivSpecificName and Locale.Lookup(sCityNameForCiv) ~= sCityNameForCiv and not IsNameUsedByCivilization(sCityNameForCiv, CivilizationTypeName) then -- means that this civilization has a specific name available for this generic city
										bestDistance = distance
										bestName = sCityNameForCiv
									elseif distance < bestDefaultDistance and not IsNameUsedOnContinent(name, mapX, mapY) then -- use generic name
										bestDefaultDistance = distance
										bestDefaultName = name
									end							
								end
							end	
						end
					end
				end
				if not bestName then
					bestName = bestDefaultName
				end
			if bestName then
				pCity:SetName(bestName)
				print("- New name : " .. tostring(bestName))
			else
				print("- Can't find a name for this position !")
				-- todo : use a name not reserved for the map
			end
		end
	end
end
Events.CityInitialized.Add( ChangeCityName )

-- function ListCityWithoutLOC()
	-- for row in GameInfo.CityMap() do
		-- local name = row.CityLocaleName
		-- if name then
			-- if Locale.Lookup(name) == name then
				-- print("WARNING : no translation for "..tostring(name))
			-- end
		-- else
			-- print("ERROR : no name at row "..tostring(row.Index + 1))
		-- end
	-- end
-- end
-- Events.LoadScreenClose.Add( ListCityWithoutLOC )

-- function ListCityNotOnMap()
	-- local hasMap = {}
	-- for row in GameInfo.CityMap() do
		-- local name = row.CityLocaleName
		-- if name then
			-- hasMap[name] = true
			-- hasMap[Locale.Lookup(name)] = true
		-- else
			-- print("ERROR : no name at row "..tostring(row.Index + 1))
		-- end
	-- end
	-- for row in GameInfo.CityNames() do
		-- local name = row.CityName
		-- local civilization = row.CivilizationType
		-- if not (hasMap[name] or hasMap[Locale.Lookup(name)]) then
			-- print("Not mapped for "..tostring(civilization).." : "..tostring(name))
		-- end
	-- end
-- end
-- Events.LoadScreenClose.Add( ListCityNotOnMap )

----------------------------------------------------------------------------------------
end
----------------------------------------------------------------------------------------
-- City renaming >>>>>
----------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------
-- EnforcingTSL <<<<<
----------------------------------------------------------------------------------------
if MapConfiguration.GetValue("ForceTSL") and MapConfiguration.GetValue("ForceTSL") ~= "FORCE_TSL_OFF" then
----------------------------------------------------------------------------------------
print("Enforcing TSL...")
function ForceTSL( iPrevPlayer )

	if Game.GetCurrentGameTurn() > GameConfiguration.GetStartTurn() then -- only called on first turn
		print("-- Removing force TSL script")
		Events.PlayerTurnDeactivated.Remove( ForceTSL )
		return
	end
	
	local bForceAll = (MapConfiguration.GetValue("ForceTSL") == "FORCE_TSL_ALL")
	local bForceAI = (MapConfiguration.GetValue("ForceTSL") == "FORCE_TSL_AI") or bForceAll
	local bForceCS = (MapConfiguration.GetValue("ForceTSL") == "FORCE_TSL_CS") or bForceAI
	local csDestroyed = 0
	
	local iPlayer = iPrevPlayer + 1
	local player = Players[iPlayer]
	if iPlayer ~= idMaori then

		if player and player:WasEverAlive() and player:GetCities():GetCount() == 0	
		   and ((not player:IsMajor() and bForceCS) or (player:IsMajor() and (not player:IsHuman()) and bForceAI) or (player:IsHuman() and bForceAll))
		   and not player:IsBarbarian() then
			if not player:IsMajor() then
				print(" - prevCounter = " .. tostring(prevCounter) .. "  csCounter = " .. tostring(csCounter))
				if (csCounter > cityStateCount) then
					-- csDestroyed = 1	
					-- UnitManager.InitUnit(player, "UNIT_SETTLER", -1, -1)
					-- local playerUnits = player:GetUnits()
					-- for i, unit in playerUnits:Members() do
						-- if(unit:GetX() >= 0 or  unit:GetY() >= 0) then
							-- print(" - City state count exceeded, destroying unit")
							-- UnitManager.Kill(unit, false)
						-- end
					-- end
					return
				else
					print("- Checking for Settler on TSL for player #".. tostring(iPlayer))
					local startingPlot = player:GetStartingPlot()
					if startingPlot and not startingPlot:IsCity() then
						local unitsInPlot = Units.GetUnitsInPlot(startingPlot)
						if unitsInPlot ~= nil then
							for _, unit in ipairs(unitsInPlot) do
								if unit:GetType() == GameInfo.Units["UNIT_SETTLER"].Index then
									print("  - found Settler !")
									print("  - create city here...")
									local city = player:GetCities():Create(startingPlot:GetX(), startingPlot:GetY())
									if city then
										print ("  - deleting settler...")
										player:GetUnits():Destroy(unit)
									else
										print ("  - WARNING : city is nil (tried to place at plot ".. tostring(startingPlot:GetX())..","..tostring(startingPlot:GetY()))
									end
								end
							end
						end		
					end	
				end				
			else
				print("- Checking for Settler on TSL for player #".. tostring(iPlayer))
				local startingPlot = player:GetStartingPlot()
				if startingPlot and not startingPlot:IsCity() then
					local unitsInPlot = Units.GetUnitsInPlot(startingPlot)
					if unitsInPlot ~= nil then
						for _, unit in ipairs(unitsInPlot) do
							if unit:GetType() == GameInfo.Units["UNIT_SETTLER"].Index then
								print("  - found Settler !")
								print("  - create city here...")
								local city = player:GetCities():Create(startingPlot:GetX(), startingPlot:GetY())
								if city then
									print ("  - deleting settler...")
									player:GetUnits():Destroy(unit)
								else
									print ("  - WARNING : city is nil (tried to place at plot ".. tostring(startingPlot:GetX())..","..tostring(startingPlot:GetY()))
								end
							end
						end
					end		
				end	
			end
		end
	
	end

end
Events.PlayerTurnDeactivated.Add( ForceTSL ) -- On TurnActivated, it seems the AI has already moved the initial settler...

function OnEnterGame()
	ForceTSL( -1 ) -- test ForceTSL on player 0
end
Events.LoadScreenClose.Add(OnEnterGame)
----------------------------------------------------------------------------------------
end
----------------------------------------------------------------------------------------
-- EnforcingTSL >>>>>
----------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------
-- Spawn Resources and Goody Huts <<<<<
----------------------------------------------------------------------------------------
function SpawnResources(CivilizationTypeName)		
				if (CivilizationTypeName == "CIVILIZATION_AUSTRALIA") then
					if (mapName == "TSLEE") then 
						print("Adding resources for Australia on " .. tostring(mapName))
						ResourceBuilder.SetResourceType(Map.GetPlot(70,10), sheep, 1)
						ResourceBuilder.SetResourceType(Map.GetPlot(71,13), sheep, 1)
						ResourceBuilder.SetResourceType(Map.GetPlot(74,7), sheep, 1)
						ResourceBuilder.SetResourceType(Map.GetPlot(75,14), sheep, 1)
						ResourceBuilder.SetResourceType(Map.GetPlot(76,9), sheep, 1)
						ResourceBuilder.SetResourceType(Map.GetPlot(77,4), sheep, 1)
						ResourceBuilder.SetResourceType(Map.GetPlot(77,6), sheep, 1)
						ResourceBuilder.SetResourceType(Map.GetPlot(77,10), horses, 1)
						ResourceBuilder.SetResourceType(Map.GetPlot(77,12), sheep, 1)
						ResourceBuilder.SetResourceType(Map.GetPlot(78,11), sheep, 1)
					end				
					if (mapName == "CordEarth") then 
						print("Adding resources for Australia on " .. tostring(mapName))
						ResourceBuilder.SetResourceType(Map.GetPlot(69,10), sheep, 1)
						ResourceBuilder.SetResourceType(Map.GetPlot(75,9), horses, 1)
						ResourceBuilder.SetResourceType(Map.GetPlot(72,10), sheep, 1)
						ResourceBuilder.SetResourceType(Map.GetPlot(76,11), sheep, 1)
						ResourceBuilder.SetResourceType(Map.GetPlot(74,7), sheep, 1)
						ResourceBuilder.SetResourceType(Map.GetPlot(75,13), sheep, 1)
						ResourceBuilder.SetResourceType(Map.GetPlot(76,11), sheep, 1)
					end
					if (mapName == "TinyEarth") then 
						print("Adding resources for Australia on " .. tostring(mapName))
						ResourceBuilder.SetResourceType(Map.GetPlot(5,1), sheep, 1)
						ResourceBuilder.SetResourceType(Map.GetPlot(6,3), horses, 1)
						ResourceBuilder.SetResourceType(Map.GetPlot(3,1), sheep, 1)
					end					
				end														
end

function GoodyHuts()
	if iAddGoodies < bAddGoodies then
		print("Adding more tribal villages to map")
		local gridWidth, gridHeight = Map.GetGridSize()
		AddGoodies(gridWidth, gridHeight)
		iAddGoodies = iAddGoodies + 1
		print("iAddGoodies = " .. tostring(iAddGoodies))
	end
end

function AddHorses()
	if bHorsesAmericas then
		if (mapName == "TSLEE") then 
			print("Adding horses in the Americas")
			ResourceBuilder.SetResourceType(Map.GetPlot(9,45), horses, 1)
			ResourceBuilder.SetResourceType(Map.GetPlot(13,47), horses, 1)
			ResourceBuilder.SetResourceType(Map.GetPlot(15,43), horses, 1)
			ResourceBuilder.SetResourceType(Map.GetPlot(14,40), horses, 1)
			
		end		
		if (mapName == "CordEarth") then 
			print("Adding horses in the Americas")
			ResourceBuilder.SetResourceType(Map.GetPlot(17,7), horses, 1)
			ResourceBuilder.SetResourceType(Map.GetPlot(8,33), horses, 1)
			ResourceBuilder.SetResourceType(Map.GetPlot(11,36), horses, 1)
			ResourceBuilder.SetResourceType(Map.GetPlot(14,39), horses, 1)
		end		
		if (mapName == "TinyEarth") then 
			print("Adding horses in the Americas")
			ResourceBuilder.SetResourceType(Map.GetPlot(21,3), horses, 1)
			ResourceBuilder.SetResourceType(Map.GetPlot(16,17), horses, 1)
			ResourceBuilder.SetResourceType(Map.GetPlot(18,18), horses, 1)
			ResourceBuilder.SetResourceType(Map.GetPlot(18,15), horses, 1)			
		end				
	end
end

function NaturalWondersSpawn()
	if not bCliffsOfDover then
		if (mapName == "CordEarth") then 
			print("Removing cliffs of Dover")
			TerrainBuilder.SetFeatureType(Map.GetPlot(34,32), -1)
			TerrainBuilder.SetTerrainType(Map.GetPlot(34,32), plains)
			TerrainBuilder.SetTerrainType(Map.GetPlot(33,32), plains)
		end	 
		if (mapName == "TSLEE") then 
			print("Removing cliffs of Dover")
			TerrainBuilder.SetFeatureType(Map.GetPlot(37,46), -1)
			TerrainBuilder.SetTerrainType(Map.GetPlot(37,46), plains)
			TerrainBuilder.SetTerrainType(Map.GetPlot(36,46), plains)
		end			
	end
	if not bDeadSea then
		if (mapName == "CordEarth") then 
			print("Removing Dead Sea")
			TerrainBuilder.SetFeatureType(Map.GetPlot(47,18), -1)
			TerrainBuilder.SetTerrainType(Map.GetPlot(47,18), desertHills)
			TerrainBuilder.SetTerrainType(Map.GetPlot(47,19), desertHills)
		end	 
		if (mapName == "TSLEE") then 
			print("Removing Dead Sea")
			TerrainBuilder.SetFeatureType(Map.GetPlot(49,33), -1)
			TerrainBuilder.SetTerrainType(Map.GetPlot(49,33), desertHills)
			TerrainBuilder.SetTerrainType(Map.GetPlot(50,34), desertHills)		
		end			
	end	
	if not bCraterLake then
		if (mapName == "CordEarth") then 
			print("Removing Crater Lake")
			TerrainBuilder.SetFeatureType(Map.GetPlot(8,40), -1)
			TerrainBuilder.SetTerrainType(Map.GetPlot(8,40), desertHills)
		end	 		
	end		
	if not bVikingDLC then
		if (mapName == "CordEarth") then 
			print("Replacing Viking DLC plots")
			TerrainBuilder.SetFeatureType(Map.GetPlot(36,37), forest)
			TerrainBuilder.SetFeatureType(Map.GetPlot(37,37), forest)
			TerrainBuilder.SetFeatureType(Map.GetPlot(37,38), forest)
		end	 
		if (mapName == "TSLEE") then 
			print("Replacing Viking DLC plots")
			TerrainBuilder.SetFeatureType(Map.GetPlot(40,53), forest)
			TerrainBuilder.SetFeatureType(Map.GetPlot(40,52), forest)
			TerrainBuilder.SetFeatureType(Map.GetPlot(41,52), forest)	
		end			
	end			
end



----------------------------------------------------------------------------------------
-- Spawn Resources and Goody Huts <<<<<<
----------------------------------------------------------------------------------------

function SpawnPlayer(iPlayer)

	if Game.GetCurrentGameTurn() > GameConfiguration.GetStartTurn() then -- Stops calling SpawnPlayer after first turn
		print(" --Removing first turn spawns script ")
		GameEvents.PlayerTurnStarted.Remove( SpawnPlayer )
		return
	end

		if iTurn <= 1 then
			if not bNoGoodies then
				print("Checking goody huts")
				GoodyHuts()
			end
			AddHorses()
			NaturalWondersSpawn()
			local player = Players[iPlayer]
			if player then
				if not player:IsBarbarian() then
					local CivilizationTypeName = PlayerConfigurations[iPlayer]:GetCivilizationTypeName()
					SpawnResources(CivilizationTypeName)
				end
			else
				print("WARNING: player is nil in SpawnPlayer #"..tostring(iPlayer))
			end
			-- if iPlayer = 0 then
				-- AddRivers()
			-- end
			print(" - Checking if player is city-state")
			if not player:IsMajor() then
				print(" - Checking if City State count is over ".. tostring(cityStateCount))
				prevCounter = csCounter
				csCounter = prevCounter + 1
				print(" - prevCounter = " .. tostring(prevCounter) .. "  csCounter = " .. tostring(csCounter))
				if (csCounter > cityStateCount) then
					-- csDestroyed = 1	
					-- UnitManager.InitUnit(player, "UNIT_SETTLER", -1, -1)
					local playerUnits = player:GetUnits()
					for i, unit in playerUnits:Members() do
						if(unit:GetX() >= 0 or  unit:GetY() >= 0) then
							print(" - City state count exceeded, destroying unit")
							UnitManager.Kill(unit, false)
						end
					end
					return
				end		
			end					
		end
end

if iTurn <= 1 then
	if (mapName == "TSLEE" or mapName == "CordEarth" or mapName == "TinyEarth") then
		GameEvents.PlayerTurnStarted.Add( SpawnPlayer )
	end
end

----------------------------------------------------------------------------------------
-- Spawn Resources and Goody Huts >>>>>>
----------------------------------------------------------------------------------------