------------------------------------------------------------------------------
--	FILE:	 TSLEE_GS.lua
--  by totalslacker
------------------------------------------------------------------------------

include "MapEnums"
include "MapUtilities"
include "MountainsCliffs"
include "RiversLakes"
include "FeatureGenerator"
include "TerrainGenerator"
include "NaturalWonderGenerator"
include "ResourceGenerator"
include "AssignStartingPlots"

local mapName = MapConfiguration.GetValue("MapName")
print("Loading "..tostring(mapName).." script")

local g_iW = 84
local g_iH = 54
local g_iFlags = {};
local g_continentsFrac = nil;

------------------------------------------------------------------------------
-- The application side will call GetMapScriptInfo directly to request
-- information about the map script.
-------------------------------------------------------------------------------
function GetMapInitData(worldSize)
	return {
		Width = g_iW,
		Height = g_iH,
		WrapX = true,
		WrapY = false,
	};      
end

-------------------------------------------------------------------------------
function GenerateMap()
	print("Calling Map Generator");
	-- Everything has been moved to the modded AssignStartingPlots in the "override" folder, search for "YnAMP" string in that file
	GenerateImportedMap(GetMap(), GetCiv6DataToConvert(), GetNaturalWonders(), g_iW, g_iH)
	
	-- Handle Nile delta flow direction (North)
	-- for x = 56, 60 do
		-- for y = 33, 37 do
			-- local plot = Map.GetPlot(x,y)
			-- if plot then
				-- MakeRiverFlowToNorth(plot)			
			-- end
		-- end
	-- end
	
	-- Handle source of the Nile flow direction	
	-- for x = 60, 61 do
		-- for y = 22, 23 do
			-- local plot = Map.GetPlot(x,y)
			-- if plot then
				-- MakeRiverFlowToNorth(plot)			
			-- end
		-- end
	-- end
	
end

function GetNaturalWonders()
	--Deprecated function, returns nothing
	local NaturalWonders = {}	
	return NaturalWonders
end

function GetCiv6DataToConvert()
	-- Map data was exported from Civ6 WB, cliffs are included in MapToConvert
	return {}
end

function GetMap()

	local MapToConvert = {}
	for i = 0, g_iW - 1, 1 do
		MapToConvert[i] = {}
	end
	
	-- Map Data (from Civ6 WB)
	-- MapToConvert[x][y] = {civ6TerrainType, civ6FeatureTypes, civ6ContinentType, {{IsNEOfRiver, flow}, {IsWOfRiver, flow}, {IsNWOfRiver, flow}}, {Civ6ResourceType, num}, {IsNEOfCliff, IsWOfCliff, IsNWOfCliff} }
MapToConvert[83][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][0]={"TERRAIN_OCEAN",-1,-1,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[46][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[19][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][0]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][1]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][1]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][1]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[46][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][1]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][1]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][1]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[19][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][1]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][2]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][2]={"TERRAIN_GRASS_HILLS","FEATURE_PIOPIOTAHI","CONTINENT_ZEALANDIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][2]={"TERRAIN_GRASS","FEATURE_PIOPIOTAHI","CONTINENT_ZEALANDIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][2]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WHALES",1},{0,0,0}}
MapToConvert[77][2]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[76][2]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[46][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][2]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][2]={"TERRAIN_TUNDRA",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,1}}
MapToConvert[23][2]={"TERRAIN_TUNDRA",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][2]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][2]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WHALES",1},{0,0,0}}
MapToConvert[20][2]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[19][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][2]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][3]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][3]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ZEALANDIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][3]={"TERRAIN_GRASS_HILLS","FEATURE_PIOPIOTAHI","CONTINENT_ZEALANDIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][3]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][3]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][3]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][3]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[74][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[46][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][3]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WHALES",1},{0,0,0}}
MapToConvert[26][3]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][3]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[23][3]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[22][3]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][3]={"TERRAIN_TUNDRA_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,1}}
MapToConvert[20][3]={"TERRAIN_TUNDRA_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
MapToConvert[19][3]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,1,0}}
MapToConvert[18][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][3]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][4]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ZEALANDIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][4]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ZEALANDIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][4]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[80][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][4]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][4]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][4]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[75][4]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[74][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[46][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][4]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[27][4]={"TERRAIN_TUNDRA",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][4]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][4]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][4]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][4]={"TERRAIN_TUNDRA","FEATURE_TORRES_DEL_PAINE","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][4]={"TERRAIN_TUNDRA","FEATURE_TORRES_DEL_PAINE","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][4]={"TERRAIN_GRASS_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[19][4]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[18][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][4]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][4]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WHALES",1},{0,0,0}}
MapToConvert[83][5]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][5]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][5]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,1}}
MapToConvert[80][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][5]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[76][5]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][5]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[74][5]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[46][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][5]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][5]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][5]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[22][5]={"TERRAIN_TUNDRA",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][5]={"TERRAIN_GRASS_MOUNTAIN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][5]={"TERRAIN_GRASS_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[19][5]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
MapToConvert[18][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][5]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][5]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][5]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][6]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ZEALANDIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][6]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][6]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][6]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
MapToConvert[76][6]={"TERRAIN_PLAINS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[75][6]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
MapToConvert[74][6]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[73][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[46][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][6]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][6]={"TERRAIN_DESERT",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][6]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SILVER",1},{0,0,0}}
MapToConvert[21][6]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][6]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
MapToConvert[19][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][6]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][6]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][6]={"TERRAIN_GRASS_MOUNTAIN","FEATURE_VOLCANO","CONTINENT_ZEALANDIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][6]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ZEALANDIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][7]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ZEALANDIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][7]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[81][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][7]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[77][7]={"TERRAIN_GRASS_MOUNTAIN",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][7]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][7]={"TERRAIN_PLAINS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][7]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[73][7]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[72][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WHALES",1},{0,0,0}}
MapToConvert[71][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][7]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][7]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[68][7]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][7]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[46][7]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][7]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][7]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][7]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[42][7]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WHALES",1},{0,0,0}}
MapToConvert[40][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][7]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][7]={"TERRAIN_DESERT",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_MERCURY",1},{0,0,0}}
MapToConvert[22][7]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][7]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COAL",1},{0,0,0}}
MapToConvert[20][7]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[19][7]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[18][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][7]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][7]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][7]={"TERRAIN_GRASS","FEATURE_GEOTHERMAL_FISSURE","CONTINENT_ZEALANDIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[83][8]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_ZEALANDIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][8]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][8]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][8]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
MapToConvert[77][8]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][8]={"TERRAIN_PLAINS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][8]={"TERRAIN_PLAINS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][8]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[73][8]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[72][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[71][8]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][8]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][8]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][8]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][8]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][8]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[46][8]={"TERRAIN_GRASS_MOUNTAIN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][8]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][8]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][8]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][8]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][8]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][8]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][8]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[23][8]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[22][8]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][8]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][8]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[19][8]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][8]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][8]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[0][8]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{1,0,1}}
MapToConvert[83][9]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][9]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[81][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][9]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[78][9]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[77][9]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COAL",1},{0,0,0}}
MapToConvert[76][9]={"TERRAIN_PLAINS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][9]={"TERRAIN_DESERT",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][9]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][9]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
MapToConvert[72][9]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[71][9]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][9]={"TERRAIN_PLAINS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][9]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][9]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][9]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][9]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[46][9]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[45][9]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][9]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][9]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][9]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][9]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][9]={"TERRAIN_PLAINS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][9]={"TERRAIN_PLAINS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][9]={"TERRAIN_PLAINS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][9]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][9]={"TERRAIN_PLAINS_MOUNTAIN","FEATURE_VOLCANO","CONTINENT_SOUTH_AMERICA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][9]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[19][9]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][9]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][10]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][10]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
MapToConvert[78][10]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][10]={"TERRAIN_PLAINS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][10]={"TERRAIN_DESERT",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][10]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][10]={"TERRAIN_DESERT",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][10]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_IRON",1},{1,0,1}}
MapToConvert[72][10]={"TERRAIN_DESERT",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
MapToConvert[71][10]={"TERRAIN_DESERT",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
MapToConvert[70][10]={"TERRAIN_PLAINS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][10]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][10]={"TERRAIN_PLAINS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][10]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[66][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][10]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][10]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COAL",1},{0,1,1}}
MapToConvert[46][10]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][10]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[44][10]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[43][10]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{1,4}},{"RESOURCE_DIAMONDS",1},{0,0,0}}
MapToConvert[42][10]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[41][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][10]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][10]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][10]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][10]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][10]={"TERRAIN_PLAINS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][10]={"TERRAIN_PLAINS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][10]={"TERRAIN_PLAINS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][10]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][10]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][10]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[19][10]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WHALES",1},{0,0,0}}
MapToConvert[18][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][10]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][11]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{1,0,0}}
MapToConvert[78][11]={"TERRAIN_PLAINS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][11]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COAL",1},{0,0,0}}
MapToConvert[76][11]={"TERRAIN_DESERT",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][11]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SILVER",1},{0,0,0}}
MapToConvert[74][11]={"TERRAIN_DESERT",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][11]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_URANIUM",1},{0,0,0}}
MapToConvert[72][11]={"TERRAIN_DESERT",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][11]={"TERRAIN_DESERT",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][11]={"TERRAIN_DESERT",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][11]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_ALUMINUM",1},{0,0,0}}
MapToConvert[68][11]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_NITER",1},{0,0,0}}
MapToConvert[67][11]={"TERRAIN_PLAINS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][11]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WHALES",1},{0,0,0}}
MapToConvert[64][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][11]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][11]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
MapToConvert[46][11]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][11]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[44][11]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][11]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CATTLE",1},{0,0,0}}
MapToConvert[42][11]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[41][11]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[40][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][11]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][11]={"TERRAIN_TUNDRA_HILLS",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][11]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][11]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[26][11]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][11]={"TERRAIN_PLAINS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][11]={"TERRAIN_PLAINS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][11]={"TERRAIN_PLAINS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][11]={"TERRAIN_PLAINS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[21][11]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][11]={"TERRAIN_PLAINS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[19][11]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][11]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][12]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][12]={"TERRAIN_PLAINS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][12]={"TERRAIN_PLAINS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][12]={"TERRAIN_DESERT",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][12]={"TERRAIN_DESERT",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][12]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][12]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COAL",1},{0,0,0}}
MapToConvert[72][12]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][12]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_GOLD",1},{0,0,0}}
MapToConvert[70][12]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_IRON",1},{0,0,0}}
MapToConvert[69][12]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][12]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][12]={"TERRAIN_DESERT",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][12]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][12]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][12]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][12]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][12]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][12]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COAL",1},{0,0,0}}
MapToConvert[46][12]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_GOLD",1},{0,0,0}}
MapToConvert[45][12]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][12]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_URANIUM",1},{0,0,0}}
MapToConvert[43][12]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][12]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
MapToConvert[41][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][12]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][12]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][12]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][12]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][12]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][12]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{1,2},{1,3},{1,1}},{"RESOURCE_TOBACCO",1},{0,0,0}}
MapToConvert[24][12]={"TERRAIN_PLAINS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][12]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_URANIUM",1},{0,0,0}}
MapToConvert[22][12]={"TERRAIN_PLAINS_MOUNTAIN",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][12]={"TERRAIN_DESERT",-1,"CONTINENT_SOUTH_AMERICA",{{1,5},{0,-1},{1,4}},{"RESOURCE_COTTON",1},{0,0,0}}
MapToConvert[20][12]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[19][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][12]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][13]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[78][13]={"TERRAIN_PLAINS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][13]={"TERRAIN_PLAINS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][13]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][13]={"TERRAIN_DESERT",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][13]={"TERRAIN_DESERT","FEATURE_GEOTHERMAL_FISSURE","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][13]={"TERRAIN_DESERT","FEATURE_OASIS","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][13]={"TERRAIN_PLAINS","FEATURE_ULURU","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][13]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][13]={"TERRAIN_DESERT",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][13]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][13]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_URANIUM",1},{0,0,0}}
MapToConvert[67][13]={"TERRAIN_DESERT",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][13]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_PEARLS",1},{0,1,0}}
MapToConvert[65][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][13]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][13]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][13]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][13]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[50][13]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[49][13]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[48][13]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][13]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[46][13]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CATTLE",1},{0,0,0}}
MapToConvert[45][13]={"TERRAIN_DESERT","FEATURE_MARSH","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][13]={"TERRAIN_DESERT","FEATURE_MARSH","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][13]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[42][13]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[41][13]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_PEARLS",1},{0,1,0}}
MapToConvert[40][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][13]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][13]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][13]={"TERRAIN_GRASS","FEATURE_PANTANAL","CONTINENT_SOUTH_AMERICA",{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[24][13]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][13]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[22][13]={"TERRAIN_PLAINS_MOUNTAIN",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][13]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][13]={"TERRAIN_DESERT",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_NITER",1},{0,0,0}}
MapToConvert[19][13]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][13]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][14]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][14]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][14]={"TERRAIN_COAST","FEATURE_BARRIER_REEF",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][14]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_BANANAS",1},{0,0,0}}
MapToConvert[77][14]={"TERRAIN_PLAINS_MOUNTAIN",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][14]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][14]={"TERRAIN_PLAINS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][14]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[73][14]={"TERRAIN_PLAINS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][14]={"TERRAIN_DESERT",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][14]={"TERRAIN_DESERT","FEATURE_OASIS","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][14]={"TERRAIN_DESERT",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][14]={"TERRAIN_DESERT",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][14]={"TERRAIN_DESERT",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][14]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
MapToConvert[66][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][14]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][14]={"TERRAIN_GRASS_MOUNTAIN","FEATURE_VOLCANO",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][14]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][14]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[52][14]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DYES",1},{0,0,0}}
MapToConvert[51][14]={"TERRAIN_DESERT_HILLS",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_ALUMINUM",1},{0,0,0}}
MapToConvert[50][14]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][14]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SUGAR",1},{0,0,0}}
MapToConvert[48][14]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_AFRICA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][14]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_AFRICA",{{1,2},{1,3},{1,1}},{"RESOURCE_COAL",1},{0,0,0}}
MapToConvert[46][14]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[45][14]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_AFRICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][14]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][14]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][14]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
MapToConvert[41][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][14]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[29][14]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][14]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][14]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COAL",1},{0,1,1}}
MapToConvert[26][14]={"TERRAIN_GRASS","FEATURE_PANTANAL","CONTINENT_SOUTH_AMERICA",{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[25][14]={"TERRAIN_GRASS","FEATURE_PANTANAL","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][14]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][14]={"TERRAIN_PLAINS_MOUNTAIN",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][14]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][14]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[20][14]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[19][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][14]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[13][14]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][14]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][15]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][15]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_NITER",1},{0,0,0}}
MapToConvert[81][15]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][15]={"TERRAIN_COAST","FEATURE_BARRIER_REEF",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][15]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COAL",1},{0,0,0}}
MapToConvert[76][15]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][15]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][15]={"TERRAIN_PLAINS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][15]={"TERRAIN_PLAINS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][15]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_URANIUM",1},{0,0,0}}
MapToConvert[71][15]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DIAMONDS",1},{0,0,0}}
MapToConvert[70][15]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_IRON",1},{0,0,0}}
MapToConvert[69][15]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][15]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_PEARLS",1},{1,0,0}}
MapToConvert[67][15]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
MapToConvert[66][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[65][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][15]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][15]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][15]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][15]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][15]={"TERRAIN_PLAINS_HILLS","FEATURE_TSINGY",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][15]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][15]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][15]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][15]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_AFRICA",{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[46][15]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_IVORY",1},{0,0,0}}
MapToConvert[45][15]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][15]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{1,3},{0,-1}},{"RESOURCE_DIAMONDS",1},{0,0,0}}
MapToConvert[43][15]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][15]={"TERRAIN_PLAINS_MOUNTAIN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][15]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][15]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][15]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][15]={"TERRAIN_OCEAN",-1,-1,{{1,5},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[30][15]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][15]={"TERRAIN_GRASS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][15]={"TERRAIN_GRASS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][15]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SUGAR",1},{0,0,0}}
MapToConvert[26][15]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_SOUTH_AMERICA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][15]={"TERRAIN_GRASS","FEATURE_PANTANAL","CONTINENT_SOUTH_AMERICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][15]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][15]={"TERRAIN_PLAINS_MOUNTAIN","FEATURE_CERRO_DE_POTOSI",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][15]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][15]={"TERRAIN_DESERT",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][15]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[19][15]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[18][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][15]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][15]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][15]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][15]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][16]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][16]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[81][16]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][16]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][16]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][16]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][16]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][16]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][16]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][16]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_ALUMINUM",1},{0,0,0}}
MapToConvert[71][16]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][16]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_PEARLS",1},{0,1,1}}
MapToConvert[69][16]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
MapToConvert[68][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][16]={"TERRAIN_COAST",-1,-1,{{0,-1},{1,3},{1,4}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[53][16]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][16]={"TERRAIN_PLAINS_HILLS",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][16]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][16]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[49][16]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][16]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}} -- Lake
MapToConvert[47][16]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[46][16]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][16]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][16]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CATTLE",1},{0,0,0}}
MapToConvert[43][16]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][16]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][16]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][16]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][16]={"TERRAIN_PLAINS_HILLS",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][16]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][16]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][16]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][16]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][16]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[27][16]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COFFEE",1},{0,0,0}}
MapToConvert[26][16]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_SOUTH_AMERICA",{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[25][16]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][16]={"TERRAIN_PLAINS_MOUNTAIN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][16]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][16]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SILVER",1},{0,0,0}}
MapToConvert[21][16]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[20][16]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[19][16]={"TERRAIN_COAST",-1,-1,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][16]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][16]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][16]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][16]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_PEARLS",1},{0,0,0}}
MapToConvert[8][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][16]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[2][16]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[1][16]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][16]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][17]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[82][17]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][17]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_TURTLES",1},{0,0,0}}
MapToConvert[80][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][17]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_TURTLES",1},{0,0,0}}
MapToConvert[77][17]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_ALUMINUM",1},{0,0,0}}
MapToConvert[76][17]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][17]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][17]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][17]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][17]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][17]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][17]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
MapToConvert[69][17]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][17]={"TERRAIN_COAST",-1,-1,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[53][17]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][17]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,1}}
MapToConvert[51][17]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
MapToConvert[50][17]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][17]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SPICES",1},{0,0,0}}
MapToConvert[48][17]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][17]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}} -- Lake
MapToConvert[46][17]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][17]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][17]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][17]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_AFRICA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][17]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][17]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[40][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][17]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][17]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][17]={"TERRAIN_COAST",-1,-1,{{1,2},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[30][17]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DYES",1},{0,0,1}}
MapToConvert[29][17]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][17]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{1,5},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][17]={"TERRAIN_PLAINS",-1,"CONTINENT_SOUTH_AMERICA",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[26][17]={"TERRAIN_PLAINS",-1,"CONTINENT_SOUTH_AMERICA",{{1,2},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[25][17]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_SOUTH_AMERICA",{{1,2},{1,3},{1,1}},{-1,1},{0,0,0}}
MapToConvert[24][17]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[23][17]={"TERRAIN_PLAINS_MOUNTAIN",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[22][17]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][17]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}} -- Lake
MapToConvert[20][17]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[19][17]={"TERRAIN_DESERT",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][17]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[17][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][17]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_TURTLES",1},{0,0,0}}
MapToConvert[9][17]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][17]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][17]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][17]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][17]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][17]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[2][17]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][17]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][17]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][18]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][18]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[81][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][18]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][18]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][18]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][18]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][18]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][18]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][18]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][18]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][18]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[53][18]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
MapToConvert[52][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][18]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][18]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][18]={"TERRAIN_GRASS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][18]={"TERRAIN_GRASS_MOUNTAIN","FEATURE_VOLCANO",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[46][18]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][18]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][18]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CATTLE",1},{0,0,0}}
MapToConvert[43][18]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_AFRICA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][18]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][18]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][18]={"TERRAIN_COAST","FEATURE_REEF",-1,{{1,2},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[30][18]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[29][18]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[28][18]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_IRON",1},{0,0,0}}
MapToConvert[27][18]={"TERRAIN_PLAINS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][18]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][18]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_SOUTH_AMERICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][18]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][18]={"TERRAIN_GRASS_MOUNTAIN",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][18]={"TERRAIN_GRASS_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][18]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][18]={"TERRAIN_DESERT",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[19][18]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COTTON",1},{0,0,0}}
MapToConvert[18][18]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][18]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][18]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[8][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][18]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[6][18]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][18]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_PEARLS",1},{0,0,0}}
MapToConvert[4][18]={"TERRAIN_PLAINS_MOUNTAIN","FEATURE_VOLCANO","CONTINENT_OCEANIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][18]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_TURTLES",1},{0,0,0}}
MapToConvert[2][18]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_PEARLS",1},{0,0,0}}
MapToConvert[1][18]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][18]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][19]={"TERRAIN_GRASS_MOUNTAIN","FEATURE_VOLCANO","CONTINENT_OCEANIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][19]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][19]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][19]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][19]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][19]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][19]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[71][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][19]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][19]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[68][19]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][19]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][19]={"TERRAIN_GRASS_MOUNTAIN","FEATURE_VOLCANO","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][19]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][19]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][19]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][19]={"TERRAIN_GRASS_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][19]={"TERRAIN_GRASS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[46][19]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][19]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][19]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[43][19]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][19]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][19]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[40][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][19]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[31][19]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][19]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{1,3},{0,-1}},{"RESOURCE_SUGAR",1},{0,0,1}}
MapToConvert[29][19]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][19]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{1,0},{0,-1}},{"RESOURCE_DIAMONDS",1},{0,0,0}}
MapToConvert[27][19]={"TERRAIN_PLAINS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][19]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][19]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][19]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][19]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][19]={"TERRAIN_GRASS_MOUNTAIN",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][19]={"TERRAIN_GRASS_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][19]={"TERRAIN_GRASS_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[19][19]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][19]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_SOUTH_AMERICA",{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[17][19]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WHALES",1},{0,0,0}}
MapToConvert[16][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][19]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][19]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][19]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][19]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][19]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][19]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][20]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][20]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][20]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_OCEANIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_GOLD",1},{0,0,0}}
MapToConvert[80][20]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][20]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][20]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_OCEANIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][20]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][20]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][20]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_PEARLS",1},{0,0,0}}
MapToConvert[71][20]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][20]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][20]={"TERRAIN_PLAINS_HILLS","FEATURE_GEOTHERMAL_FISSURE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][20]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][20]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[66][20]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][20]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[64][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][20]={"TERRAIN_OCEAN",-1,-1,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[62][20]={"TERRAIN_OCEAN",-1,-1,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[61][20]={"TERRAIN_OCEAN",-1,-1,{{1,2},{1,3},{1,1}},{-1,1},{0,0,0}}
MapToConvert[60][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[59][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][20]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][20]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[51][20]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[50][20]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][20]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][20]={"TERRAIN_GRASS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CATTLE",1},{0,0,0}}
MapToConvert[47][20]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}} -- Lake
MapToConvert[46][20]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COFFEE",1},{0,0,0}}
MapToConvert[45][20]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][20]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DIAMONDS",1},{0,0,0}}
MapToConvert[43][20]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_BANANAS",1},{0,0,0}}
MapToConvert[42][20]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][20]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][20]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][20]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][20]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][20]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][20]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[28][20]={"TERRAIN_PLAINS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][20]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_NITER",1},{0,0,0}}
MapToConvert[26][20]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][20]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_GOLD",1},{0,0,0}}
MapToConvert[24][20]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][20]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{1,0},{1,1}},{"RESOURCE_DYES",1},{0,0,0}}
MapToConvert[22][20]={"TERRAIN_GRASS_MOUNTAIN",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][20]={"TERRAIN_GRASS_MOUNTAIN",-1,-1,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][20]={"TERRAIN_GRASS_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[19][20]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[18][20]={"TERRAIN_DESERT",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][20]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][20]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[5][20]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][20]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][20]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][20]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][21]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][21]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_OCEANIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][21]={"TERRAIN_GRASS_MOUNTAIN",-1,"CONTINENT_OCEANIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][21]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_OCEANIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][21]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_OCEANIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][21]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][21]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[72][21]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][21]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_STONE",1},{0,0,0}}
MapToConvert[70][21]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][21]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_STONE",1},{0,0,0}}
MapToConvert[68][21]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][21]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][21]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][21]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][21]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[62][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][21]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][21]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][21]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][21]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][21]={"TERRAIN_PLAINS_MOUNTAIN","FEATURE_KILIMANJARO",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][21]={"TERRAIN_GRASS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[46][21]={"TERRAIN_GRASS_MOUNTAIN","FEATURE_VOLCANO","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][21]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][21]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][21]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][21]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_AFRICA",{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[41][21]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[40][21]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][21]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WHALES",1},{0,0,0}}
MapToConvert[37][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][21]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][21]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DYES",1},{0,0,0}}
MapToConvert[30][21]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][21]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{1,1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[28][21]={"TERRAIN_PLAINS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][21]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][21]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][21]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[24][21]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][21]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[22][21]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][21]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{1,5},{0,-1},{0,-1}},{"RESOURCE_COFFEE",1},{0,0,0}}
MapToConvert[20][21]={"TERRAIN_GRASS_MOUNTAIN",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[19][21]={"TERRAIN_GRASS_MOUNTAIN",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[18][21]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][21]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][21]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[15][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][21]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][21]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_PEARLS",1},{0,0,0}}
MapToConvert[3][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][21]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][22]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][22]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][22]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][22]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_OCEANIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][22]={"TERRAIN_GRASS_MOUNTAIN",-1,"CONTINENT_OCEANIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][22]={"TERRAIN_GRASS_MOUNTAIN",-1,"CONTINENT_OCEANIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][22]={"TERRAIN_GRASS_MOUNTAIN",-1,"CONTINENT_OCEANIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][22]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][22]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][22]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][22]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][22]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[71][22]={"TERRAIN_COAST",-1,-1,{{0,-1},{1,3},{1,1}},{-1,1},{0,0,0}}
MapToConvert[70][22]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[69][22]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][22]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[67][22]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][22]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][22]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_STONE",1},{1,0,0}}
MapToConvert[64][22]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{1,4}},{-1,1},{0,1,0}}
MapToConvert[63][22]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][22]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[61][22]={"TERRAIN_OCEAN",-1,-1,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][22]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][22]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][22]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_TURTLES",1},{0,0,0}}
MapToConvert[57][22]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][22]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][22]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][22]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][22]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[52][22]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][22]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_INCENSE",1},{0,0,0}}
MapToConvert[50][22]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][22]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][22]={"TERRAIN_GRASS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CATTLE",1},{0,0,0}}
MapToConvert[47][22]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}} -- Lake
MapToConvert[46][22]={"TERRAIN_GRASS_MOUNTAIN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][22]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_AFRICA",{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[44][22]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_IVORY",1},{0,0,0}}
MapToConvert[43][22]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][22]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][22]={"TERRAIN_GRASS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][22]={"TERRAIN_GRASS_MOUNTAIN","FEATURE_VOLCANO","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][22]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][22]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][22]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][22]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][22]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][22]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][22]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][22]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[31][22]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[30][22]={"TERRAIN_PLAINS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][22]={"TERRAIN_PLAINS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][22]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_URANIUM",1},{0,0,0}}
MapToConvert[27][22]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_ALUMINUM",1},{0,0,0}}
MapToConvert[26][22]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][22]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][22]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_SOUTH_AMERICA",{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[23][22]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][22]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[21][22]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[20][22]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[19][22]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[18][22]={"TERRAIN_DESERT",-1,"CONTINENT_SOUTH_AMERICA",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[17][22]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][22]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][22]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][22]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][22]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][22]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][22]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][22]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][22]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][22]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[7][22]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][22]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][22]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][22]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][22]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][22]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][22]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][22]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][23]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][23]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][23]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][23]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][23]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_OCEANIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][23]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_OCEANIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][23]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_OCEANIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_BANANAS",1},{0,0,0}}
MapToConvert[76][23]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_OCEANIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][23]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_OCEANIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][23]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_PEARLS",1},{0,0,0}}
MapToConvert[73][23]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][23]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{1,5},{0,-1},{1,4}},{"RESOURCE_STONE",1},{0,0,0}}
MapToConvert[71][23]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[70][23]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][23]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][23]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][23]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][23]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][23]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][23]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[63][23]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[62][23]={"TERRAIN_OCEAN",-1,-1,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][23]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][23]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][23]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][23]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][23]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_PEARLS",1},{0,0,0}}
MapToConvert[56][23]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][23]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][23]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][23]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][23]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_INCENSE",1},{0,0,0}}
MapToConvert[51][23]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][23]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][23]={"TERRAIN_DESERT_MOUNTAIN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][23]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][23]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_NITER",1},{0,0,0}}
MapToConvert[46][23]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_AFRICA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][23]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][23]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][23]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][23]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{1,3},{1,4}},{"RESOURCE_BANANAS",1},{0,0,0}}
MapToConvert[41][23]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_IVORY",1},{0,0,0}}
MapToConvert[40][23]={"TERRAIN_GRASS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][23]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][23]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][23]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][23]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][23]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][23]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][23]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][23]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][23]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{1,0,0}}
MapToConvert[30][23]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
MapToConvert[29][23]={"TERRAIN_PLAINS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][23]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][23]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][23]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][23]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[24][23]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{1,2},{0,-1},{1,1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[23][23]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{1,2},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][23]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[21][23]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][23]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[19][23]={"TERRAIN_GRASS_MOUNTAIN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][23]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COAL",1},{0,0,0}}
MapToConvert[17][23]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][23]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][23]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_TURTLES",1},{0,0,0}}
MapToConvert[14][23]={"TERRAIN_COAST","FEATURE_GALAPAGOS",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][23]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][23]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][23]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][23]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][23]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][23]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_PEARLS",1},{0,0,0}}
MapToConvert[7][23]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][23]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][23]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][23]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][23]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[2][23]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][23]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][23]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][24]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][24]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][24]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][24]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][24]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][24]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][24]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][24]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_OCEANIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SPICES",1},{0,0,0}}
MapToConvert[75][24]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[74][24]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][24]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SPICES",1},{0,0,0}}
MapToConvert[72][24]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][24]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_STONE",1},{0,0,0}}
MapToConvert[70][24]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_BANANAS",1},{0,0,0}}
MapToConvert[69][24]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][24]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][24]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][24]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][24]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][24]={"TERRAIN_GRASS_MOUNTAIN","FEATURE_VOLCANO","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[63][24]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[62][24]={"TERRAIN_COAST",-1,-1,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][24]={"TERRAIN_COAST",-1,-1,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][24]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{1,4}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[59][24]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][24]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][24]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][24]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][24]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][24]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[53][24]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][24]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][24]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][24]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][24]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[48][24]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COFFEE",1},{0,0,0}}
MapToConvert[47][24]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_AFRICA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[46][24]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_AFRICA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][24]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][24]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[43][24]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_AFRICA",{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[42][24]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_ALUMINUM",1},{0,0,0}}
MapToConvert[41][24]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][24]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[39][24]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][24]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[37][24]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][24]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][24]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[34][24]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][24]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][24]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][24]={"TERRAIN_OCEAN",-1,-1,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[30][24]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[29][24]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][24]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[27][24]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[26][24]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{1,1}},{"RESOURCE_ALUMINUM",1},{0,0,0}}
MapToConvert[25][24]={"TERRAIN_GRASS_MOUNTAIN","FEATURE_RORAIMA",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][24]={"TERRAIN_GRASS_MOUNTAIN","FEATURE_RORAIMA",-1,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][24]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][24]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][24]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{1,3},{1,1}},{-1,1},{0,0,0}}
MapToConvert[20][24]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{1,0},{1,1}},{"RESOURCE_COFFEE",1},{0,0,0}}
MapToConvert[19][24]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][24]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[17][24]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][24]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][24]={"TERRAIN_COAST","FEATURE_GALAPAGOS",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][24]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][24]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][24]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][24]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][24]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][24]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][24]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][24]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[6][24]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][24]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][24]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][24]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][24]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][24]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][24]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][25]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][25]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[74][25]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
MapToConvert[73][25]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SPICES",1},{0,0,0}}
MapToConvert[72][25]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][25]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_ALUMINUM",1},{0,0,0}}
MapToConvert[70][25]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_IRON",1},{0,0,0}}
MapToConvert[69][25]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_STONE",1},{0,0,0}}
MapToConvert[68][25]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_BANANAS",1},{0,0,0}}
MapToConvert[67][25]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][25]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[65][25]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][25]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][25]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[62][25]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][25]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SPICES",1},{0,0,0}}
MapToConvert[60][25]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][25]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[57][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][25]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][25]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][25]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[50][25]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][25]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_AFRICA",{{0,-1},{0,-1},{1,1}},{"RESOURCE_SALT",1},{0,0,0}}
MapToConvert[48][25]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][25]={"TERRAIN_GRASS_MOUNTAIN",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[46][25]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_AFRICA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][25]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_AFRICA",{{0,-1},{1,0},{0,-1}},{"RESOURCE_IVORY",1},{0,0,0}}
MapToConvert[44][25]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][25]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_BANANAS",1},{0,0,0}}
MapToConvert[42][25]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][25]={"TERRAIN_PLAINS_MOUNTAIN",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][25]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][25]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_AFRICA",{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[38][25]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_ALUMINUM",1},{0,0,0}}
MapToConvert[37][25]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][25]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][25]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COFFEE",1},{0,0,0}}
MapToConvert[34][25]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{1,5},{0,-1},{1,4}},{"RESOURCE_SPICES",1},{0,0,0}}
MapToConvert[33][25]={"TERRAIN_COAST",-1,-1,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[32][25]={"TERRAIN_OCEAN",-1,-1,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[31][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[30][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][25]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][25]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SPICES",1},{0,0,0}}
MapToConvert[25][25]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][25]={"TERRAIN_GRASS_MOUNTAIN","FEATURE_RORAIMA",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][25]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][25]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_SOUTH_AMERICA",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[21][25]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_SOUTH_AMERICA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][25]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{1,1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[19][25]={"TERRAIN_GRASS_MOUNTAIN",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][25]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][25]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][25]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][26]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[81][26]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[80][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][26]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][26]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][26]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][26]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][26]={"TERRAIN_GRASS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][26]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][26]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_BANANAS",1},{0,0,0}}
MapToConvert[69][26]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][26]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][26]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][26]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][26]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][26]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][26]={"TERRAIN_COAST",-1,-1,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][26]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_PEARLS",1},{0,0,0}}
MapToConvert[60][26]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SPICES",1},{1,0,0}}
MapToConvert[59][26]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[58][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][26]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[52][26]={"TERRAIN_GRASS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][26]={"TERRAIN_GRASS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][26]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_PEARLS",1},{1,0,0}}
MapToConvert[49][26]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][26]={"TERRAIN_PLAINS_MOUNTAIN","FEATURE_VOLCANO","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][26]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[46][26]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_AFRICA",{{1,5},{0,-1},{0,-1}},{"RESOURCE_CATTLE",1},{0,0,0}}
MapToConvert[45][26]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_AFRICA",{{1,2},{1,0},{1,1}},{"RESOURCE_CATTLE",1},{0,0,0}}
MapToConvert[44][26]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][26]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_IVORY",1},{0,0,0}}
MapToConvert[42][26]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][26]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_AFRICA",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[40][26]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{1,2},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[39][26]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][26]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_IVORY",1},{0,0,0}}
MapToConvert[37][26]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_NITER",1},{0,0,0}}
MapToConvert[36][26]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{1,0},{0,-1}},{"RESOURCE_GOLD",1},{0,0,0}}
MapToConvert[35][26]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][26]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][26]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[32][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][26]={"TERRAIN_COAST",-1,-1,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][26]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][26]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][26]={"TERRAIN_GRASS_MOUNTAIN","FEATURE_RORAIMA",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][26]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][26]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_SOUTH_AMERICA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][26]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][26]={"TERRAIN_GRASS_MOUNTAIN",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[19][26]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][26]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[17][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][26]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][26]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][26]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][27]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][27]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][27]={"TERRAIN_PLAINS","FEATURE_JUNGLE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][27]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][27]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][27]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_OCEANIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][27]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][27]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][27]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][27]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][27]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][27]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][27]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][27]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][27]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[68][27]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][27]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][27]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][27]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][27]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][27]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][27]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][27]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][27]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][27]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[58][27]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][27]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][27]={"TERRAIN_OCEAN",-1,-1,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][27]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][27]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[53][27]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][27]={"TERRAIN_GRASS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][27]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COFFEE",1},{0,0,0}}
MapToConvert[50][27]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][27]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][27]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][27]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[46][27]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][27]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[44][27]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][27]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][27]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CATTLE",1},{0,0,0}}
MapToConvert[41][27]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}} -- Lake
MapToConvert[40][27]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][27]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][27]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][27]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][27]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[35][27]={"TERRAIN_PLAINS_MOUNTAIN",-1,-1,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][27]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][27]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][27]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][27]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][27]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][27]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WHALES",1},{0,0,0}}
MapToConvert[28][27]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][27]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][27]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[25][27]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][27]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][27]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_ALUMINUM",1},{0,0,0}}
MapToConvert[22][27]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_SOUTH_AMERICA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][27]={"TERRAIN_PLAINS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][27]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_SOUTH_AMERICA",{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[19][27]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][27]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][27]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][27]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][27]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][27]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][27]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][27]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][27]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][27]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][27]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][27]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][27]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][27]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][27]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][27]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][27]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][27]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][27]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][27]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[83][28]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[82][28]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][28]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][28]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][28]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][28]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][28]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][28]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][28]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][28]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][28]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][28]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][28]={"TERRAIN_GRASS_MOUNTAIN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][28]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][28]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[68][28]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][28]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[66][28]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][28]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][28]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][28]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][28]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][28]={"TERRAIN_COAST",-1,-1,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][28]={"TERRAIN_GRASS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][28]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][28]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[57][28]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[56][28]={"TERRAIN_OCEAN",-1,-1,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][28]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][28]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][28]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][28]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][28]={"TERRAIN_DESERT_MOUNTAIN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][28]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][28]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][28]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[47][28]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{1,5},{0,-1},{0,-1}},{"RESOURCE_GOLD",1},{0,0,0}}
MapToConvert[46][28]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[45][28]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][28]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_GYPSUM",1},{0,0,0}}
MapToConvert[43][28]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][28]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][28]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][28]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][28]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{1,2},{0,-1},{0,-1}},{"RESOURCE_COTTON",1},{0,0,0}}
MapToConvert[38][28]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_AFRICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][28]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[36][28]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][28]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{1,5},{0,-1},{1,4}},{"RESOURCE_ALUMINUM",1},{0,0,0}}
MapToConvert[34][28]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[33][28]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[32][28]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[31][28]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][28]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][28]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][28]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][28]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][28]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][28]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[24][28]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][28]={"TERRAIN_PLAINS",-1,"CONTINENT_SOUTH_AMERICA",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[22][28]={"TERRAIN_PLAINS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{1,1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[21][28]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{1,5},{0,-1},{0,-1}},{"RESOURCE_COFFEE",1},{0,0,0}}
MapToConvert[20][28]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_SOUTH_AMERICA",{{0,-1},{1,0},{0,-1}},{"RESOURCE_COAL",1},{0,0,0}}
MapToConvert[19][28]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][28]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][28]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][28]={"TERRAIN_GRASS_MOUNTAIN","FEATURE_VOLCANO","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][28]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][28]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][28]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][28]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][28]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][28]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][28]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][28]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][28]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][28]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][28]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][28]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][28]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][28]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[1][28]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][28]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][29]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][29]={"TERRAIN_PLAINS","FEATURE_JUNGLE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][29]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][29]={"TERRAIN_PLAINS","FEATURE_JUNGLE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][29]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[78][29]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][29]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][29]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][29]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][29]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][29]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][29]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][29]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][29]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{1,4}},{"RESOURCE_PEARLS",1},{0,0,0}}
MapToConvert[69][29]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][29]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][29]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{"RESOURCE_CATTLE",1},{0,0,0}}
MapToConvert[66][29]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][29]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_STONE",1},{0,0,0}}
MapToConvert[64][29]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][29]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][29]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][29]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_PEARLS",1},{0,0,0}}
MapToConvert[60][29]={"TERRAIN_GRASS","FEATURE_FLOODPLAINS_GRASSLAND","CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{"RESOURCE_RICE",1},{0,0,0}}
MapToConvert[59][29]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][29]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_GOLD",1},{1,0,0}}
MapToConvert[57][29]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][29]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][29]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[54][29]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][29]={"TERRAIN_DESERT","FEATURE_OASIS","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][29]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_INCENSE",1},{0,0,0}}
MapToConvert[51][29]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][29]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][29]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][29]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{1,0,0}}
MapToConvert[47][29]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[46][29]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][29]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][29]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][29]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][29]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][29]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SALT",1},{0,0,0}}
MapToConvert[40][29]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_URANIUM",1},{0,0,0}}
MapToConvert[39][29]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][29]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_AFRICA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][29]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_AFRICA",{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[36][29]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SALT",1},{0,0,0}}
MapToConvert[35][29]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][29]={"TERRAIN_DESERT","FEATURE_EYE_OF_THE_SAHARA","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][29]={"TERRAIN_DESERT","FEATURE_EYE_OF_THE_SAHARA","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][29]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][29]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][29]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][29]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][29]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][29]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][29]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][29]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][29]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][29]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[22][29]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][29]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][29]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_SOUTH_AMERICA",{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[19][29]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][29]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][29]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_STONE",1},{0,0,0}}
MapToConvert[16][29]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_GOLD",1},{0,0,0}}
MapToConvert[15][29]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[14][29]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][29]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][29]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][29]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][29]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][29]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][29]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][29]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][29]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][29]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][29]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][29]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][29]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][29]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][29]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][30]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[82][30]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][30]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][30]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[79][30]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][30]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][30]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][30]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_GOLD",1},{0,0,0}}
MapToConvert[75][30]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][30]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[73][30]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][30]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][30]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][30]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][30]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][30]={"TERRAIN_PLAINS","FEATURE_FLOODPLAINS_PLAINS","CONTINENT_ASIA",{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[67][30]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][30]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][30]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[64][30]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{"RESOURCE_RICE",1},{0,0,0}}
MapToConvert[63][30]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[62][30]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][30]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][30]={"TERRAIN_GRASS","FEATURE_FLOODPLAINS_GRASSLAND","CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{"RESOURCE_SUGAR",1},{0,0,0}}
MapToConvert[59][30]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[58][30]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][30]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,1,0}}
MapToConvert[56][30]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][30]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[54][30]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_INCENSE",1},{0,0,0}}
MapToConvert[53][30]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][30]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[51][30]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][30]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_STONE",1},{0,0,0}}
MapToConvert[49][30]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SHEEP",1},{0,0,0}}
MapToConvert[48][30]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][30]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[46][30]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_AFRICA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][30]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_AFRICA",{{0,-1},{1,0},{0,-1}},{"RESOURCE_COTTON",1},{0,0,0}}
MapToConvert[44][30]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_STONE",1},{0,0,0}}
MapToConvert[43][30]={"TERRAIN_DESERT_MOUNTAIN",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][30]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][30]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][30]={"TERRAIN_DESERT_MOUNTAIN",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][30]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][30]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][30]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][30]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][30]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][30]={"TERRAIN_DESERT","FEATURE_EYE_OF_THE_SAHARA","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][30]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][30]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][30]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][30]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][30]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][30]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][30]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[26][30]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][30]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[24][30]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][30]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][30]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[21][30]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[20][30]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[19][30]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_TURTLES",1},{0,0,0}}
MapToConvert[18][30]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][30]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][30]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][30]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][30]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[13][30]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][30]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][30]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][30]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][30]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][30]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][30]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][30]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][30]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][30]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][30]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][30]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][30]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][30]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][31]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][31]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][31]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][31]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][31]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][31]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][31]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][31]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][31]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][31]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][31]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[72][31]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][31]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][31]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][31]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[68][31]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[67][31]={"TERRAIN_PLAINS","FEATURE_FLOODPLAINS_PLAINS","CONTINENT_ASIA",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[66][31]={"TERRAIN_GRASS",-1,"CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][31]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][31]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][31]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][31]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][31]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[60][31]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{"RESOURCE_ALUMINUM",1},{0,1,0}}
MapToConvert[59][31]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][31]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][31]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][31]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[55][31]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][31]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][31]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[52][31]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[51][31]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][31]={"TERRAIN_DESERT_HILLS","FEATURE_OASIS","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][31]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][31]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][31]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{1,1,0}}
MapToConvert[46][31]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_IRON",1},{0,1,0}}
MapToConvert[45][31]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_AFRICA",{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[44][31]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_STONE",1},{0,0,0}}
MapToConvert[43][31]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_STONE",1},{0,0,0}}
MapToConvert[42][31]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][31]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_IVORY",1},{0,0,0}}
MapToConvert[40][31]={"TERRAIN_DESERT","FEATURE_OASIS","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][31]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][31]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][31]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SALT",1},{0,0,0}}
MapToConvert[36][31]={"TERRAIN_DESERT","FEATURE_OASIS","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][31]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][31]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][31]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][31]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][31]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[30][31]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][31]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][31]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][31]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][31]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][31]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][31]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][31]={"TERRAIN_COAST",-1,-1,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[22][31]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][31]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][31]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[19][31]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][31]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][31]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_STONE",1},{0,0,0}}
MapToConvert[16][31]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_STONE",1},{0,0,0}}
MapToConvert[15][31]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{1,0},{0,-1}},{"RESOURCE_JADE",1},{0,0,0}}
MapToConvert[14][31]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_STONE",1},{1,0,1}}
MapToConvert[13][31]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
MapToConvert[12][31]={"TERRAIN_GRASS_MOUNTAIN","FEATURE_VOLCANO","CONTINENT_NORTH_AMERICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][31]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[10][31]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][31]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][31]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][31]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][31]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][31]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][31]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][31]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][31]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][31]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][31]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][32]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][32]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][32]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][32]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][32]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][32]={"TERRAIN_PLAINS","FEATURE_JUNGLE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][32]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][32]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][32]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][32]={"TERRAIN_GRASS","FEATURE_GEOTHERMAL_FISSURE","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][32]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][32]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][32]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][32]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][32]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[68][32]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COAL",1},{0,1,0}}
MapToConvert[67][32]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][32]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][32]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][32]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[63][32]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[62][32]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][32]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_BANANAS",1},{0,0,1}}
MapToConvert[60][32]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][32]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_ASIA",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[58][32]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[57][32]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][32]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][32]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[54][32]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][32]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_PEARLS",1},{0,0,0}}
MapToConvert[52][32]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[51][32]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][32]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SALT",1},{0,0,0}}
MapToConvert[49][32]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_HORSES",1},{0,0,0}}
MapToConvert[48][32]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][32]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[46][32]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_AFRICA",{{1,5},{0,-1},{0,-1}},{"RESOURCE_WHEAT",1},{0,0,0}}
MapToConvert[45][32]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_AFRICA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][32]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][32]={"TERRAIN_DESERT","FEATURE_OASIS","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][32]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][32]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[40][32]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][32]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][32]={"TERRAIN_DESERT","FEATURE_OASIS","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][32]={"TERRAIN_DESERT",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][32]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_IRON",1},{0,0,0}}
MapToConvert[35][32]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][32]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][32]={"TERRAIN_DESERT_MOUNTAIN",-1,-1,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[32][32]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][32]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][32]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][32]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][32]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][32]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][32]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_TURTLES",1},{0,0,0}}
MapToConvert[25][32]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][32]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SUGAR",1},{0,0,0}}
MapToConvert[23][32]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COFFEE",1},{0,0,0}}
MapToConvert[22][32]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_PEARLS",1},{0,0,0}}
MapToConvert[21][32]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[20][32]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_ALUMINUM",1},{0,0,0}}
MapToConvert[19][32]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][32]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][32]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][32]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][32]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{1,0},{0,-1}},{"RESOURCE_COCOA",1},{0,0,0}}
MapToConvert[14][32]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][32]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[12][32]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[11][32]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[10][32]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][32]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][32]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][32]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][32]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][32]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][32]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][32]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][32]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][32]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][32]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][33]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][33]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][33]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][33]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][33]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][33]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][33]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][33]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][33]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][33]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][33]={"TERRAIN_PLAINS_MOUNTAIN","FEATURE_VOLCANO",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][33]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][33]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][33]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[69][33]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
MapToConvert[68][33]={"TERRAIN_COAST","FEATURE_HA_LONG_BAY",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][33]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][33]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][33]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[64][33]={"TERRAIN_PLAINS","FEATURE_FLOODPLAINS_PLAINS","CONTINENT_ASIA",{{1,2},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][33]={"TERRAIN_GRASS_MOUNTAIN",-1,"CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][33]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][33]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_ASIA",{{1,2},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][33]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_ASIA",{{1,2},{0,-1},{1,1}},{"RESOURCE_CITRUS",1},{0,0,0}}
MapToConvert[59][33]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_IVORY",1},{0,0,0}}
MapToConvert[58][33]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{1,5},{0,-1},{1,4}},{"RESOURCE_IRON",1},{0,0,0}}
MapToConvert[57][33]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[56][33]={"TERRAIN_DESERT","FEATURE_MARSH","CONTINENT_ASIA",{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[55][33]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][33]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][33]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][33]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][33]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][33]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][33]={"TERRAIN_COAST","FEATURE_DEAD_SEA",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}} -- Lake
MapToConvert[48][33]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][33]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[46][33]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WHEAT",1},{0,0,0}}
MapToConvert[45][33]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_AFRICA",{{1,5},{1,3},{1,4}},{"RESOURCE_RICE",1},{0,0,0}}
MapToConvert[44][33]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_AFRICA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][33]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][33]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][33]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,1,0}}
MapToConvert[40][33]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][33]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][33]={"TERRAIN_DESERT_MOUNTAIN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][33]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_MARBLE",1},{0,0,0}}
MapToConvert[36][33]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][33]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][33]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][33]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][33]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[31][33]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][33]={"TERRAIN_PLAINS_MOUNTAIN","FEATURE_VOLCANO","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][33]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][33]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][33]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][33]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][33]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][33]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[23][33]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][33]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][33]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SUGAR",1},{0,0,0}}
MapToConvert[20][33]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[19][33]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][33]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][33]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][33]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][33]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[14][33]={"TERRAIN_PLAINS_MOUNTAIN",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][33]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][33]={"TERRAIN_GRASS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WHEAT",1},{0,0,0}}
MapToConvert[11][33]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[10][33]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][33]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][33]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][33]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][33]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][33]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][33]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][33]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][33]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][33]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][33]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][34]={"TERRAIN_OCEAN","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][34]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][34]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][34]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][34]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][34]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][34]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][34]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][34]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][34]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_AUSTRALIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][34]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][34]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][34]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[70][34]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][34]={"TERRAIN_COAST","FEATURE_HA_LONG_BAY",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][34]={"TERRAIN_PLAINS","FEATURE_FLOODPLAINS_PLAINS","CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[67][34]={"TERRAIN_PLAINS","FEATURE_FLOODPLAINS_PLAINS","CONTINENT_ASIA",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[66][34]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][34]={"TERRAIN_PLAINS_MOUNTAIN",-1,-1,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[64][34]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[63][34]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{"RESOURCE_BANANAS",1},{0,0,0}}
MapToConvert[62][34]={"TERRAIN_GRASS","FEATURE_FLOODPLAINS_GRASSLAND","CONTINENT_ASIA",{{1,2},{1,3},{1,1}},{"RESOURCE_RICE",1},{0,0,0}}
MapToConvert[61][34]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][34]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COAL",1},{0,0,0}}
MapToConvert[59][34]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][34]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][34]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_ASIA",{{1,5},{1,3},{1,4}},{"RESOURCE_WHEAT",1},{0,0,0}}
MapToConvert[56][34]={"TERRAIN_DESERT_MOUNTAIN",-1,-1,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[55][34]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_GYPSUM",1},{0,0,0}}
MapToConvert[54][34]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CITRUS",1},{0,0,0}}
MapToConvert[53][34]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[52][34]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[51][34]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][34]={"TERRAIN_COAST","FEATURE_DEAD_SEA",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}} -- Lake
MapToConvert[49][34]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][34]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][34]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[46][34]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][34]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[44][34]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][34]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[42][34]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
MapToConvert[41][34]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][34]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WHEAT",1},{0,0,0}}
MapToConvert[39][34]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][34]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][34]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][34]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][34]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][34]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][34]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][34]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][34]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][34]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][34]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][34]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][34]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][34]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][34]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][34]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][34]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][34]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][34]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COCOA",1},{0,0,0}}
MapToConvert[20][34]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[19][34]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_TOBACCO",1},{0,0,0}}
MapToConvert[18][34]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][34]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[16][34]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][34]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[14][34]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{1,5},{0,-1},{0,-1}},{"RESOURCE_SILVER",1},{0,0,0}}
MapToConvert[13][34]={"TERRAIN_PLAINS_MOUNTAIN",-1,"CONTINENT_NORTH_AMERICA",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[12][34]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[11][34]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][34]={"TERRAIN_DESERT_MOUNTAIN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][34]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WHALES",1},{0,0,0}}
MapToConvert[8][34]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][34]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][34]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][34]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][34]={"TERRAIN_PLAINS_MOUNTAIN","FEATURE_VOLCANO","CONTINENT_OCEANIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][34]={"TERRAIN_PLAINS","FEATURE_JUNGLE","CONTINENT_OCEANIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][34]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][34]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][34]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][35]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][35]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][35]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][35]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][35]={"TERRAIN_OCEAN","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][35]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][35]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][35]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][35]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][35]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][35]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][35]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][35]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
MapToConvert[70][35]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_PEARLS",1},{0,0,0}}
MapToConvert[69][35]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][35]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][35]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][35]={"TERRAIN_GRASS","FEATURE_FLOODPLAINS_GRASSLAND","CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][35]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{1,2},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][35]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{"RESOURCE_JADE",1},{0,0,0}}
MapToConvert[63][35]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{1,5},{1,3},{1,4}},{"RESOURCE_COAL",1},{0,0,0}}
MapToConvert[62][35]={"TERRAIN_GRASS","FEATURE_FLOODPLAINS_GRASSLAND","CONTINENT_ASIA",{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[61][35]={"TERRAIN_GRASS","FEATURE_FLOODPLAINS_GRASSLAND","CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{"RESOURCE_WHEAT",1},{0,0,0}}
MapToConvert[60][35]={"TERRAIN_GRASS","FEATURE_FLOODPLAINS_GRASSLAND","CONTINENT_ASIA",{{1,2},{0,-1},{1,1}},{"RESOURCE_WHEAT",1},{0,0,0}}
MapToConvert[59][35]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{"RESOURCE_NITER",1},{0,0,0}}
MapToConvert[58][35]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][35]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{"RESOURCE_WHEAT",1},{0,0,0}}
MapToConvert[56][35]={"TERRAIN_DESERT_MOUNTAIN",-1,-1,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][35]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][35]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][35]={"TERRAIN_GRASS_MOUNTAIN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][35]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{"RESOURCE_WHEAT",1},{0,0,0}}
MapToConvert[51][35]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{"RESOURCE_WHEAT",1},{0,0,0}}
MapToConvert[50][35]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WHEAT",1},{0,0,0}}
MapToConvert[49][35]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DYES",1},{0,0,0}}
MapToConvert[48][35]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,1,0}}
MapToConvert[47][35]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{1,1,0}}
MapToConvert[46][35]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,1,0}}
MapToConvert[45][35]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[44][35]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][35]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][35]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[41][35]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WINE",1},{0,0,0}}
MapToConvert[40][35]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][35]={"TERRAIN_PLAINS",-1,"CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][35]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][35]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_AFRICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][35]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,0}}
MapToConvert[35][35]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,1}}
MapToConvert[34][35]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][35]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][35]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][35]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][35]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][35]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][35]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][35]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][35]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][35]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][35]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][35]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][35]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][35]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_TURTLES",1},{0,0,0}}
MapToConvert[20][35]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[19][35]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[18][35]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][35]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][35]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[15][35]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][35]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][35]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[12][35]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][35]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_TOBACCO",1},{0,0,0}}
MapToConvert[10][35]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_PEARLS",1},{0,0,0}}
MapToConvert[9][35]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][35]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][35]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][35]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][35]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][35]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][35]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SUGAR",1},{0,0,0}}
MapToConvert[2][35]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[1][35]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][35]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][36]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][36]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][36]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][36]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][36]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][36]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][36]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][36]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][36]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][36]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][36]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][36]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
MapToConvert[71][36]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][36]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][36]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][36]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{1,2},{0,-1},{1,1}},{"RESOURCE_CITRUS",1},{0,0,0}}
MapToConvert[67][36]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][36]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][36]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{1,2},{1,3},{0,-1}},{"RESOURCE_ALUMINUM",1},{0,0,0}}
MapToConvert[64][36]={"TERRAIN_GRASS_MOUNTAIN",-1,"CONTINENT_ASIA",{{1,2},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][36]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][36]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][36]={"TERRAIN_GRASS_MOUNTAIN","FEATURE_EVEREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][36]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][36]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{"RESOURCE_COTTON",1},{0,0,0}}
MapToConvert[58][36]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COTTON",1},{0,0,0}}
MapToConvert[57][36]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_ASIA",{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[56][36]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][36]={"TERRAIN_DESERT","FEATURE_OASIS","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][36]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][36]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][36]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_ASIA",{{1,2},{1,3},{1,4}},{"RESOURCE_COTTON",1},{0,0,0}}
MapToConvert[51][36]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{"RESOURCE_WHEAT",1},{0,0,0}}
MapToConvert[50][36]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][36]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,1,1}}
MapToConvert[48][36]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[47][36]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[46][36]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
MapToConvert[45][36]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_MARBLE",1},{1,0,0}}
MapToConvert[44][36]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[43][36]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][36]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{0,-1},{1,0},{0,4}},{"RESOURCE_WHEAT",1},{0,0,0}}
MapToConvert[41][36]={"TERRAIN_PLAINS_HILLS","FEATURE_VESUVIUS","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][36]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[39][36]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[38][36]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
MapToConvert[37][36]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,1}}
MapToConvert[36][36]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_NITER",1},{1,0,1}}
MapToConvert[35][36]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][36]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_EUROPE",{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[33][36]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{1,3},{0,-1}},{"RESOURCE_WINE",1},{0,0,0}}
MapToConvert[32][36]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[31][36]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][36]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[29][36]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][36]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][36]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][36]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][36]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][36]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][36]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][36]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][36]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][36]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[19][36]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][36]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[17][36]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][36]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][36]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][36]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][36]={"TERRAIN_DESERT_MOUNTAIN",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][36]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][36]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][36]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][36]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][36]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][36]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][36]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][36]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][36]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][36]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][36]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][36]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][36]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_TURTLES",1},{0,0,0}}
MapToConvert[83][37]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][37]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][37]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][37]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][37]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][37]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][37]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][37]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][37]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WHALES",1},{0,0,0}}
MapToConvert[74][37]={"TERRAIN_GRASS_MOUNTAIN","FEATURE_VOLCANO",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][37]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[72][37]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][37]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][37]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][37]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][37]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SILK",1},{0,0,0}}
MapToConvert[67][37]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{"RESOURCE_TEA",1},{0,0,0}}
MapToConvert[66][37]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{1,2},{0,-1},{1,1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[65][37]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_IRON",1},{0,0,0}}
MapToConvert[64][37]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{1,3},{1,4}},{"RESOURCE_BANANAS",1},{0,0,0}}
MapToConvert[63][37]={"TERRAIN_GRASS_MOUNTAIN",-1,"CONTINENT_ASIA",{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[62][37]={"TERRAIN_GRASS_MOUNTAIN",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][37]={"TERRAIN_GRASS_MOUNTAIN","FEATURE_EVEREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][37]={"TERRAIN_GRASS_MOUNTAIN","FEATURE_EVEREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][37]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[58][37]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DYES",1},{0,0,0}}
MapToConvert[57][37]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_ASIA",{{1,2},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[56][37]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][37]={"TERRAIN_GRASS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_HORSES",1},{0,0,0}}
MapToConvert[54][37]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][37]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[52][37]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][37]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_ASIA",{{1,2},{1,3},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[50][37]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_ASIA",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[49][37]={"TERRAIN_GRASS",-1,"CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][37]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DEER",1},{1,0,1}}
MapToConvert[47][37]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
MapToConvert[46][37]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][37]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][37]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_ALUMINUM",1},{0,0,0}}
MapToConvert[43][37]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[42][37]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][37]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[40][37]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[39][37]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[38][37]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[37][37]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][37]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,1,0}}
MapToConvert[35][37]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][37]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WHEAT",1},{0,0,0}}
MapToConvert[33][37]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[32][37]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][37]={"TERRAIN_COAST",-1,-1,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[30][37]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[29][37]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WHALES",1},{0,0,0}}
MapToConvert[28][37]={"TERRAIN_GRASS","FEATURE_FOREST",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][37]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][37]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][37]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][37]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][37]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][37]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][37]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][37]={"TERRAIN_GRASS","FEATURE_JUNGLE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[19][37]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][37]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][37]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[16][37]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][37]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][37]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][37]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][37]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_IRON",1},{0,0,0}}
MapToConvert[11][37]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_GOLD",1},{0,0,0}}
MapToConvert[10][37]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][37]={"TERRAIN_DESERT_HILLS","FEATURE_GEOTHERMAL_FISSURE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][37]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][37]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][37]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][37]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][37]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][37]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][37]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][37]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_PEARLS",1},{0,0,0}}
MapToConvert[0][37]={"TERRAIN_PLAINS_HILLS","FEATURE_JUNGLE","CONTINENT_OCEANIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_BANANAS",1},{0,0,0}}
MapToConvert[83][38]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][38]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][38]={"TERRAIN_OCEAN","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][38]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][38]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][38]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][38]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][38]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[75][38]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[74][38]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COAL",1},{1,0,0}}
MapToConvert[73][38]={"TERRAIN_COAST","FEATURE_REEF",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_PEARLS",1},{0,1,0}}
MapToConvert[72][38]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][38]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][38]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][38]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[68][38]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_JADE",1},{0,0,0}}
MapToConvert[67][38]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][38]={"TERRAIN_GRASS","FEATURE_FLOODPLAINS_GRASSLAND","CONTINENT_ASIA",{{1,2},{1,0},{1,1}},{"RESOURCE_RICE",1},{0,0,0}}
MapToConvert[65][38]={"TERRAIN_GRASS_MOUNTAIN",-1,"CONTINENT_ASIA",{{1,2},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][38]={"TERRAIN_TUNDRA_HILLS",-1,"CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][38]={"TERRAIN_TUNDRA_HILLS",-1,"CONTINENT_ASIA",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[62][38]={"TERRAIN_TUNDRA_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[61][38]={"TERRAIN_GRASS_MOUNTAIN",-1,"CONTINENT_ASIA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][38]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][38]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][38]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{1,3},{1,4}},{"RESOURCE_HORSES",1},{0,0,0}}
MapToConvert[57][38]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{1,3},{1,4}},{"RESOURCE_SHEEP",1},{0,0,0}}
MapToConvert[56][38]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_IRON",1},{0,0,0}}
MapToConvert[55][38]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][38]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][38]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][38]={"TERRAIN_GRASS_MOUNTAIN",-1,"CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][38]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][38]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[49][38]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][38]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[47][38]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_ALUMINUM",1},{0,0,0}}
MapToConvert[46][38]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{1,1,0}}
MapToConvert[45][38]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WINE",1},{0,0,0}}
MapToConvert[44][38]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_IRON",1},{0,0,0}}
MapToConvert[43][38]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[42][38]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[41][38]={"TERRAIN_GRASS_HILLS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][38]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[39][38]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SHEEP",1},{1,1,0}}
MapToConvert[38][38]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[37][38]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[36][38]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_HORSES",1},{0,0,0}}
MapToConvert[35][38]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][38]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[33][38]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][38]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[31][38]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][38]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][38]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][38]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][38]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][38]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][38]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][38]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][38]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][38]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[21][38]={"TERRAIN_GRASS","FEATURE_FLOODPLAINS_GRASSLAND","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][38]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[19][38]={"TERRAIN_GRASS","FEATURE_FLOODPLAINS_GRASSLAND","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][38]={"TERRAIN_GRASS","FEATURE_FLOODPLAINS_GRASSLAND","CONTINENT_NORTH_AMERICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][38]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[16][38]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][38]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][38]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{1,2},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[13][38]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[12][38]={"TERRAIN_DESERT_MOUNTAIN",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][38]={"TERRAIN_DESERT",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][38]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[9][38]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_URANIUM",1},{0,0,0}}
MapToConvert[8][38]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[7][38]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][38]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][38]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][38]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][38]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][38]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][38]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][38]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][39]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][39]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][39]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][39]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][39]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][39]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WHALES",1},{0,0,0}}
MapToConvert[77][39]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[76][39]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][39]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[74][39]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[73][39]={"TERRAIN_GRASS_HILLS","FEATURE_JUNGLE","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][39]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][39]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][39]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][39]={"TERRAIN_GRASS",-1,"CONTINENT_ASIA",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[68][39]={"TERRAIN_GRASS","FEATURE_FLOODPLAINS_GRASSLAND","CONTINENT_ASIA",{{1,2},{0,-1},{1,1}},{"RESOURCE_RICE",1},{0,0,0}}
MapToConvert[67][39]={"TERRAIN_GRASS","FEATURE_FLOODPLAINS_GRASSLAND","CONTINENT_ASIA",{{1,2},{0,-1},{1,1}},{"RESOURCE_RICE",1},{0,0,0}}
MapToConvert[66][39]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{1,1}},{"RESOURCE_COAL",1},{0,0,0}}
MapToConvert[65][39]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][39]={"TERRAIN_GRASS_MOUNTAIN",-1,"CONTINENT_ASIA",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[63][39]={"TERRAIN_TUNDRA_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{"RESOURCE_SHEEP",1},{0,0,0}}
MapToConvert[62][39]={"TERRAIN_TUNDRA_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_INCENSE",1},{0,0,0}}
MapToConvert[61][39]={"TERRAIN_TUNDRA_HILLS",-1,"CONTINENT_ASIA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][39]={"TERRAIN_GRASS_MOUNTAIN",-1,"CONTINENT_ASIA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][39]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[58][39]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[57][39]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{1,3},{1,4}},{"RESOURCE_DIAMONDS",1},{0,0,0}}
MapToConvert[56][39]={"TERRAIN_GRASS_MOUNTAIN",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][39]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][39]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][39]={"TERRAIN_GRASS_MOUNTAIN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][39]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[51][39]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][39]={"TERRAIN_GRASS_MOUNTAIN",-1,"CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][39]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{"RESOURCE_SILVER",1},{0,0,0}}
MapToConvert[48][39]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][39]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[46][39]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][39]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{1,2},{0,-1},{0,-1}},{"RESOURCE_SILVER",1},{0,0,1}}
MapToConvert[44][39]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[43][39]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][39]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{1,1,0}}
MapToConvert[41][39]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_MARBLE",1},{0,1,0}}
MapToConvert[40][39]={"TERRAIN_GRASS_HILLS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{1,4}},{"RESOURCE_CATTLE",1},{0,0,0}}
MapToConvert[39][39]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][39]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[37][39]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][39]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,1}}
MapToConvert[35][39]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][39]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][39]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][39]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[31][39]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][39]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][39]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][39]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][39]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][39]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][39]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][39]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][39]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][39]={"TERRAIN_GRASS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][39]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{1,2},{0,-1},{0,-1}},{"RESOURCE_TOBACCO",1},{0,0,0}}
MapToConvert[20][39]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[19][39]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{1,3},{0,-1}},{"RESOURCE_NITER",1},{0,0,0}}
MapToConvert[18][39]={"TERRAIN_GRASS","FEATURE_FLOODPLAINS_GRASSLAND","CONTINENT_NORTH_AMERICA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][39]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][39]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COTTON",1},{0,0,0}}
MapToConvert[15][39]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][39]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][39]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COTTON",1},{0,0,0}}
MapToConvert[12][39]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[11][39]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][39]={"TERRAIN_DESERT","FEATURE_MARSH","CONTINENT_NORTH_AMERICA",{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[9][39]={"TERRAIN_DESERT",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][39]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][39]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][39]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[5][39]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][39]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][39]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][39]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][39]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][39]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][40]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][40]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][40]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][40]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][40]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][40]={"TERRAIN_GRASS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][40]={"TERRAIN_GRASS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][40]={"TERRAIN_GRASS_MOUNTAIN",-1,"CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][40]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[74][40]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{"RESOURCE_RICE",1},{0,0,0}}
MapToConvert[73][40]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[72][40]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[71][40]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][40]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][40]={"TERRAIN_GRASS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][40]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][40]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_NITER",1},{0,0,0}}
MapToConvert[66][40]={"TERRAIN_GRASS","FEATURE_FLOODPLAINS_GRASSLAND","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_RICE",1},{0,0,0}}
MapToConvert[65][40]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][40]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_JADE",1},{0,0,0}}
MapToConvert[63][40]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][40]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][40]={"TERRAIN_TUNDRA_MOUNTAIN",-1,"CONTINENT_ASIA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][40]={"TERRAIN_PLAINS_MOUNTAIN",-1,"CONTINENT_ASIA",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[59][40]={"TERRAIN_PLAINS_MOUNTAIN",-1,"CONTINENT_ASIA",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[58][40]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{1,4}},{"RESOURCE_WHEAT",1},{0,0,0}}
MapToConvert[57][40]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][40]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][40]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{1,5},{0,-1},{1,4}},{"RESOURCE_NITER",1},{0,0,0}}
MapToConvert[54][40]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,1}} -- Lake
MapToConvert[53][40]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}} -- Lake
MapToConvert[52][40]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][40]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{1,5},{0,-1},{1,4}},{"RESOURCE_IRON",1},{0,0,0}}
MapToConvert[50][40]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[49][40]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][40]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][40]={"TERRAIN_GRASS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[46][40]={"TERRAIN_GRASS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][40]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][40]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][40]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[42][40]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{1,1,0}}
MapToConvert[41][40]={"TERRAIN_GRASS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][40]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_STONE",1},{0,0,0}}
MapToConvert[39][40]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][40]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_EUROPE",{{0,-1},{1,3},{0,-1}},{"RESOURCE_NITER",1},{0,0,0}}
MapToConvert[37][40]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WHEAT",1},{0,0,0}}
MapToConvert[36][40]={"TERRAIN_PLAINS_MOUNTAIN",-1,"CONTINENT_EUROPE",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][40]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][40]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_IRON",1},{0,0,0}}
MapToConvert[33][40]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[32][40]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[31][40]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][40]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][40]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][40]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][40]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][40]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][40]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][40]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[23][40]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][40]={"TERRAIN_GRASS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][40]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][40]={"TERRAIN_GRASS_MOUNTAIN",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[19][40]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][40]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{1,2},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[17][40]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[16][40]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[15][40]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[14][40]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][40]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][40]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][40]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{1,2},{0,-1},{0,-1}},{"RESOURCE_DYES",1},{0,0,0}}
MapToConvert[10][40]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_NORTH_AMERICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][40]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][40]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][40]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][40]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[5][40]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][40]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][40]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][40]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][40]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][40]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][41]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][41]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][41]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][41]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][41]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][41]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[77][41]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_ASIA",{{1,2},{0,-1},{1,1}},{"RESOURCE_RICE",1},{0,0,0}}
MapToConvert[76][41]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_ASIA",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[75][41]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][41]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][41]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][41]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][41]={"TERRAIN_GRASS","FEATURE_MARSH",-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_RICE",1},{0,0,0}}
MapToConvert[70][41]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[69][41]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][41]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_IRON",1},{0,1,0}}
MapToConvert[67][41]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SILK",1},{0,0,0}}
MapToConvert[66][41]={"TERRAIN_GRASS","FEATURE_FLOODPLAINS_GRASSLAND","CONTINENT_ASIA",{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[65][41]={"TERRAIN_GRASS","FEATURE_FLOODPLAINS_GRASSLAND","CONTINENT_EUROPE",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][41]={"TERRAIN_PLAINS","FEATURE_FLOODPLAINS_PLAINS","CONTINENT_EUROPE",{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[63][41]={"TERRAIN_DESERT_MOUNTAIN","FEATURE_ZHANGYE_DANXIA","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][41]={"TERRAIN_DESERT_MOUNTAIN","FEATURE_ZHANGYE_DANXIA","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][41]={"TERRAIN_DESERT_MOUNTAIN","FEATURE_ZHANGYE_DANXIA","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][41]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][41]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][41]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][41]={"TERRAIN_PLAINS","FEATURE_FLOODPLAINS_PLAINS","CONTINENT_ASIA",{{1,5},{0,-1},{1,4}},{"RESOURCE_WHEAT",1},{0,0,0}}
MapToConvert[56][41]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_ASIA",{{1,5},{0,-1},{1,4}},{"RESOURCE_COTTON",1},{0,0,0}}
MapToConvert[55][41]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_ASIA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][41]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[53][41]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}} -- Lake
MapToConvert[52][41]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][41]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_ASIA",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[50][41]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][41]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}} -- Lake
MapToConvert[48][41]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}} -- Lake
MapToConvert[47][41]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{1,0,1}} -- Lake
MapToConvert[46][41]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}} -- Lake
MapToConvert[45][41]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][41]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[43][41]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][41]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][41]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_EUROPE",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][41]={"TERRAIN_GRASS",-1,"CONTINENT_EUROPE",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[39][41]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[38][41]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{1,2},{0,-1},{0,-1}},{"RESOURCE_STONE",1},{0,0,0}}
MapToConvert[37][41]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{1,3},{0,-1}},{"RESOURCE_TRUFFLES",1},{0,0,0}}
MapToConvert[36][41]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][41]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WINE",1},{0,0,0}}
MapToConvert[34][41]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
MapToConvert[33][41]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{1,0,1}}
MapToConvert[32][41]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][41]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][41]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][41]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][41]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][41]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][41]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][41]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][41]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WHALES",1},{0,0,0}}
MapToConvert[23][41]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][41]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][41]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{1,1}},{"RESOURCE_DEER",1},{0,0,0}}
MapToConvert[20][41]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COAL",1},{0,0,0}}
MapToConvert[19][41]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][41]={"TERRAIN_GRASS","FEATURE_FLOODPLAINS_GRASSLAND","CONTINENT_NORTH_AMERICA",{{1,2},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[17][41]={"TERRAIN_GRASS","FEATURE_FLOODPLAINS_GRASSLAND","CONTINENT_NORTH_AMERICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][41]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][41]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DLV_BISON",1},{0,0,0}}
MapToConvert[14][41]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][41]={"TERRAIN_PLAINS_MOUNTAIN",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][41]={"TERRAIN_PLAINS_MOUNTAIN",-1,-1,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[11][41]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][41]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_NORTH_AMERICA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][41]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][41]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][41]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][41]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][41]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][41]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][41]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][41]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][41]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][41]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][42]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][42]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][42]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][42]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][42]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][42]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][42]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][42]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_IRON",1},{0,0,0}}
MapToConvert[75][42]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,1}}
MapToConvert[74][42]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,1}}
MapToConvert[73][42]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][42]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{"RESOURCE_JADE",1},{0,0,0}}
MapToConvert[71][42]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][42]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][42]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[68][42]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,1}}
MapToConvert[67][42]={"TERRAIN_PLAINS","FEATURE_FLOODPLAINS_PLAINS","CONTINENT_ASIA",{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[66][42]={"TERRAIN_PLAINS","FEATURE_FLOODPLAINS_PLAINS","CONTINENT_ASIA",{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[65][42]={"TERRAIN_PLAINS","FEATURE_FLOODPLAINS_PLAINS","CONTINENT_ASIA",{{1,5},{0,-1},{0,-1}},{"RESOURCE_SHEEP",1},{0,0,0}}
MapToConvert[64][42]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{1,0},{0,-1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[63][42]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][42]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][42]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WHEAT",1},{0,0,0}}
MapToConvert[60][42]={"TERRAIN_PLAINS_MOUNTAIN",-1,"CONTINENT_ASIA",{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[59][42]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_ASIA",{{1,5},{0,-1},{1,4}},{"RESOURCE_WHEAT",1},{0,0,0}}
MapToConvert[58][42]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_ASIA",{{1,5},{1,3},{1,4}},{"RESOURCE_IRON",1},{0,0,0}}
MapToConvert[57][42]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_ASIA",{{0,-1},{1,0},{0,-1}},{"RESOURCE_NITER",1},{0,0,0}}
MapToConvert[56][42]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_ASIA",{{1,5},{0,-1},{0,-1}},{"RESOURCE_WHEAT",1},{0,0,0}}
MapToConvert[55][42]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][42]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][42]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}} -- Lake
MapToConvert[52][42]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,1,0}}
MapToConvert[51][42]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][42]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][42]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}} -- Lake
MapToConvert[48][42]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}} -- Lake
MapToConvert[47][42]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}} -- Lake
MapToConvert[46][42]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_EUROPE",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][42]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[44][42]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][42]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][42]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][42]={"TERRAIN_GRASS_MOUNTAIN",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][42]={"TERRAIN_GRASS_MOUNTAIN","FEATURE_MATTERHORN","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][42]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[38][42]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[37][42]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[36][42]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][42]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][42]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][42]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][42]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][42]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][42]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][42]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][42]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][42]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][42]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][42]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[24][42]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][42]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[22][42]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][42]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{1,5},{0,-1},{0,-1}},{"RESOURCE_COAL",1},{0,0,0}}
MapToConvert[20][42]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[19][42]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[18][42]={"TERRAIN_GRASS","FEATURE_FLOODPLAINS_GRASSLAND","CONTINENT_NORTH_AMERICA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][42]={"TERRAIN_PLAINS","FEATURE_FLOODPLAINS_PLAINS","CONTINENT_NORTH_AMERICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][42]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][42]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][42]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][42]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{1,2},{0,-1},{0,-1}},{"RESOURCE_GOLD",1},{0,0,0}}
MapToConvert[12][42]={"TERRAIN_PLAINS_MOUNTAIN",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][42]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_NORTH_AMERICA",{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[10][42]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[9][42]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][42]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][42]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][42]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[5][42]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][42]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][42]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][42]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][42]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][42]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][43]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][43]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][43]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][43]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][43]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][43]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[77][43]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][43]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_NITER",1},{0,0,0}}
MapToConvert[75][43]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,1}}
MapToConvert[74][43]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][43]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WHALES",1},{0,0,0}}
MapToConvert[72][43]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{1,0,0}}
MapToConvert[71][43]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COAL",1},{0,1,0}}
MapToConvert[70][43]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][43]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
MapToConvert[68][43]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][43]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_HORSES",1},{0,0,0}}
MapToConvert[66][43]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{"RESOURCE_SHEEP",1},{0,0,0}}
MapToConvert[65][43]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[64][43]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[63][43]={"TERRAIN_DESERT","FEATURE_OASIS","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][43]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_INCENSE",1},{0,0,0}}
MapToConvert[61][43]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[60][43]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_ASIA",{{0,-1},{1,0},{1,1}},{"RESOURCE_WHEAT",1},{0,0,0}}
MapToConvert[59][43]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[58][43]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[57][43]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][43]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_ASIA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][43]={"TERRAIN_DESERT","FEATURE_FLOODPLAINS","CONTINENT_ASIA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][43]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][43]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[52][43]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}} -- Lake
MapToConvert[51][43]={"TERRAIN_GRASS_MOUNTAIN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][43]={"TERRAIN_GRASS_MOUNTAIN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][43]={"TERRAIN_GRASS_MOUNTAIN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][43]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}} -- Lake
MapToConvert[47][43]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
MapToConvert[46][43]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][43]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][43]={"TERRAIN_GRASS_HILLS","FEATURE_GEOTHERMAL_FISSURE","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][43]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{1,2},{0,-1},{0,-1}},{"RESOURCE_HORSES",1},{0,0,0}}
MapToConvert[42][43]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{1,2},{0,-1},{1,1}},{"RESOURCE_SALT",1},{0,0,0}}
MapToConvert[41][43]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{1,2},{0,-1},{1,1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[40][43]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{1,1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[39][43]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{1,0},{0,-1}},{"RESOURCE_IRON",1},{0,0,0}}
MapToConvert[38][43]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][43]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][43]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[35][43]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[34][43]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[33][43]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[32][43]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][43]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][43]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][43]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][43]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][43]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][43]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][43]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][43]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[23][43]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][43]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][43]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[20][43]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}} -- Lake
MapToConvert[19][43]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][43]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][43]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_NORTH_AMERICA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][43]={"TERRAIN_PLAINS","FEATURE_FLOODPLAINS_PLAINS","CONTINENT_NORTH_AMERICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][43]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][43]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DLV_BISON",1},{0,0,0}}
MapToConvert[13][43]={"TERRAIN_PLAINS_MOUNTAIN",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][43]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][43]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[10][43]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SILVER",1},{0,0,0}}
MapToConvert[9][43]={"TERRAIN_PLAINS","FEATURE_YOSEMITE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][43]={"TERRAIN_PLAINS","FEATURE_YOSEMITE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][43]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][43]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][43]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][43]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][43]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][43]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][43]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][43]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][44]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][44]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][44]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][44]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][44]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][44]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COAL",1},{1,1,1}}
MapToConvert[77][44]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][44]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{1,1}},{-1,1},{1,0,0}}
MapToConvert[75][44]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][44]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][44]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][44]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[71][44]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_IRON",1},{0,0,0}}
MapToConvert[70][44]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][44]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[68][44]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][44]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][44]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][44]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SALT",1},{0,0,0}}
MapToConvert[64][44]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[63][44]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][44]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_ASIA",{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[61][44]={"TERRAIN_PLAINS_MOUNTAIN",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[60][44]={"TERRAIN_PLAINS_MOUNTAIN",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][44]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][44]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][44]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][44]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[55][44]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}} -- Lake
MapToConvert[54][44]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][44]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}} -- Lake
MapToConvert[52][44]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[51][44]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][44]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[49][44]={"TERRAIN_GRASS_HILLS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[48][44]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{1,5},{1,3},{0,-1}},{"RESOURCE_WHEAT",1},{0,0,0}}
MapToConvert[47][44]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{1,2},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[46][44]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{1,3},{0,-1}},{"RESOURCE_COAL",1},{0,0,0}}
MapToConvert[45][44]={"TERRAIN_GRASS_MOUNTAIN",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][44]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_GOLD",1},{0,0,0}}
MapToConvert[43][44]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CATTLE",1},{0,0,0}}
MapToConvert[42][44]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][44]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][44]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{0,-1},{1,0},{1,1}},{"RESOURCE_HORSES",1},{0,0,0}}
MapToConvert[39][44]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][44]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_EUROPE",{{0,-1},{1,0},{0,-1}},{"RESOURCE_WHEAT",1},{0,0,0}}
MapToConvert[37][44]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][44]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][44]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_STONE",1},{0,0,0}}
MapToConvert[34][44]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[33][44]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][44]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][44]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][44]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][44]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][44]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][44]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][44]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][44]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][44]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][44]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{1,2},{0,-1},{0,-1}},{"RESOURCE_IRON",1},{0,0,0}}
MapToConvert[22][44]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][44]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}} -- Lake
MapToConvert[20][44]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[19][44]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{1,0},{0,-1}},{"RESOURCE_IRON",1},{0,0,0}}
MapToConvert[18][44]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}} -- Lake
MapToConvert[17][44]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][44]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[15][44]={"TERRAIN_PLAINS","FEATURE_FLOODPLAINS_PLAINS","CONTINENT_NORTH_AMERICA",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[14][44]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][44]={"TERRAIN_PLAINS","FEATURE_DEVILSTOWER","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][44]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][44]={"TERRAIN_DESERT_HILLS","FEATURE_DELICATE_ARCH","CONTINENT_NORTH_AMERICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][44]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][44]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][44]={"TERRAIN_PLAINS_HILLS","FEATURE_GEOTHERMAL_FISSURE","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][44]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][44]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[5][44]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][44]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][44]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][44]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][44]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][44]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][45]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][45]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][45]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][45]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][45]={"TERRAIN_TUNDRA_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][45]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[77][45]={"TERRAIN_TUNDRA_MOUNTAIN",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][45]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DEER",1},{0,0,0}}
MapToConvert[75][45]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
MapToConvert[74][45]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][45]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[72][45]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
MapToConvert[71][45]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COAL",1},{0,0,1}}
MapToConvert[70][45]={"TERRAIN_GRASS_MOUNTAIN",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][45]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][45]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[67][45]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_NITER",1},{0,0,0}}
MapToConvert[66][45]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][45]={"TERRAIN_DESERT","FEATURE_OASIS","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][45]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][45]={"TERRAIN_DESERT_MOUNTAIN",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][45]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_ASIA",{{1,5},{0,-1},{0,-1}},{"RESOURCE_JADE",1},{0,0,0}}
MapToConvert[61][45]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[60][45]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}} -- Lake
MapToConvert[59][45]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[58][45]={"TERRAIN_DESERT_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_URANIUM",1},{0,0,0}}
MapToConvert[57][45]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][45]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_MERCURY",1},{0,0,0}}
MapToConvert[55][45]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][45]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_HORSES",1},{0,0,0}}
MapToConvert[53][45]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][45]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{1,2},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[51][45]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[50][45]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][45]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{1,5},{1,3},{1,4}},{"RESOURCE_COAL",1},{0,0,0}}
MapToConvert[48][45]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{1,4}},{"RESOURCE_WHEAT",1},{0,0,0}}
MapToConvert[47][45]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{1,5},{1,3},{1,4}},{"RESOURCE_WHEAT",1},{0,0,0}}
MapToConvert[46][45]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_EUROPE",{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[45][45]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][45]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[43][45]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{1,5},{0,-1},{0,-1}},{"RESOURCE_DYES",1},{0,0,0}}
MapToConvert[42][45]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{1,5},{1,0},{0,-1}},{"RESOURCE_IRON",1},{0,0,0}}
MapToConvert[41][45]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][45]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{1,5},{0,-1},{0,-1}},{"RESOURCE_COAL",1},{0,0,0}}
MapToConvert[39][45]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_EUROPE",{{0,-1},{1,0},{0,-1}},{"RESOURCE_CATTLE",1},{0,0,0}}
MapToConvert[38][45]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][45]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][45]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][45]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][45]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][45]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[32][45]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][45]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][45]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][45]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][45]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][45]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][45]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][45]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[24][45]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][45]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DEER",1},{0,0,0}}
MapToConvert[22][45]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][45]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[20][45]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[19][45]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}} -- Lake
MapToConvert[18][45]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}} -- Lake
MapToConvert[17][45]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][45]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SALT",1},{0,0,0}}
MapToConvert[15][45]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][45]={"TERRAIN_PLAINS","FEATURE_FLOODPLAINS_PLAINS","CONTINENT_NORTH_AMERICA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][45]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{1,3},{0,-1}},{"RESOURCE_GYPSUM",1},{0,0,0}}
MapToConvert[12][45]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DLV_BISON",1},{0,0,0}}
MapToConvert[11][45]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[10][45]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DLV_BISON",1},{0,0,0}}
MapToConvert[9][45]={"TERRAIN_PLAINS","FEATURE_FLOODPLAINS_PLAINS","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][45]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][45]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DEER",1},{0,0,0}}
MapToConvert[6][45]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][45]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][45]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][45]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][45]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][45]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][45]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][46]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][46]={"TERRAIN_TUNDRA_MOUNTAIN","FEATURE_VOLCANO",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][46]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][46]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[79][46]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][46]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][46]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[76][46]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][46]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][46]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[73][46]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COAL",1},{0,0,0}}
MapToConvert[72][46]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][46]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DEER",1},{0,0,0}}
MapToConvert[70][46]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][46]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_GYPSUM",1},{0,0,0}}
MapToConvert[68][46]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COAL",1},{0,0,0}}
MapToConvert[67][46]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][46]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][46]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_ASIA",{{1,2},{1,0},{1,1}},{"RESOURCE_SHEEP",1},{0,0,0}}
MapToConvert[64][46]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_ASIA",{{1,2},{0,-1},{1,1}},{"RESOURCE_HORSES",1},{0,0,0}}
MapToConvert[63][46]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][46]={"TERRAIN_PLAINS_MOUNTAIN",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][46]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{1,0},{1,1}},{"RESOURCE_GOLD",1},{0,0,0}}
MapToConvert[60][46]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][46]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][46]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_HORSES",1},{0,0,0}}
MapToConvert[57][46]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_ALUMINUM",1},{0,0,0}}
MapToConvert[56][46]={"TERRAIN_DESERT",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][46]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][46]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][46]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][46]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][46]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][46]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{1,2},{1,3},{0,-1}},{"RESOURCE_HORSES",1},{0,0,0}}
MapToConvert[49][46]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][46]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{1,2},{0,-1},{0,-1}},{"RESOURCE_NITER",1},{0,0,0}}
MapToConvert[47][46]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[46][46]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_EUROPE",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][46]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_EUROPE",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][46]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][46]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][46]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{1,5},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][46]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{1,0},{0,-1}},{"RESOURCE_DEER",1},{0,0,0}}
MapToConvert[40][46]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_EUROPE",{{1,5},{0,-1},{0,-1}},{"RESOURCE_NITER",1},{0,0,0}}
MapToConvert[39][46]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][46]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][46]={"TERRAIN_PLAINS_HILLS","FEATURE_CLIFFS_DOVER","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,1}}
MapToConvert[36][46]={"TERRAIN_PLAINS_HILLS","FEATURE_CLIFFS_DOVER","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
MapToConvert[35][46]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[34][46]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][46]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][46]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][46]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][46]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][46]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][46]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WHALES",1},{0,0,0}}
MapToConvert[27][46]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[26][46]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][46]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][46]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][46]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_ALUMINUM",1},{0,1,0}}
MapToConvert[22][46]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[21][46]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][46]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FURS",1},{0,0,0}}
MapToConvert[19][46]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][46]={"TERRAIN_COAST",-1,-1,{{0,-1},{1,3},{1,1}},{"RESOURCE_FISH",1},{0,0,0}} -- Lake
MapToConvert[17][46]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][46]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_IRON",1},{0,0,0}}
MapToConvert[15][46]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][46]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{1,2},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[13][46]={"TERRAIN_PLAINS","FEATURE_FLOODPLAINS_PLAINS","CONTINENT_NORTH_AMERICA",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[12][46]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{1,3},{0,-1}},{"RESOURCE_JADE",1},{0,0,0}}
MapToConvert[11][46]={"TERRAIN_PLAINS_MOUNTAIN",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][46]={"TERRAIN_PLAINS","FEATURE_FLOODPLAINS_PLAINS","CONTINENT_NORTH_AMERICA",{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[9][46]={"TERRAIN_GRASS_MOUNTAIN","FEATURE_VOLCANO","CONTINENT_NORTH_AMERICA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][46]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[7][46]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][46]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][46]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][46]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][46]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][46]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][46]={"TERRAIN_TUNDRA_MOUNTAIN","FEATURE_VOLCANO","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][46]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][47]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][47]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][47]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[80][47]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{1,0},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[79][47]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[78][47]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][47]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][47]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][47]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[74][47]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COAL",1},{1,1,0}}
MapToConvert[73][47]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[72][47]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[71][47]={"TERRAIN_GRASS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][47]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[69][47]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][47]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][47]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_ASIA",{{0,-1},{1,0},{1,1}},{"RESOURCE_HORSES",1},{0,0,0}}
MapToConvert[66][47]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][47]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{1,1,0}} -- Lake
MapToConvert[64][47]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][47]={"TERRAIN_PLAINS_MOUNTAIN",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][47]={"TERRAIN_PLAINS_MOUNTAIN",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][47]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_ASIA",{{1,5},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[60][47]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{1,0},{0,-1}},{"RESOURCE_HORSES",1},{0,0,0}}
MapToConvert[59][47]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COAL",1},{0,0,0}}
MapToConvert[58][47]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{1,5},{0,-1},{1,4}},{"RESOURCE_WHEAT",1},{0,0,0}}
MapToConvert[57][47]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[56][47]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{1,0},{0,-1}},{"RESOURCE_WHEAT",1},{0,0,0}}
MapToConvert[55][47]={"TERRAIN_PLAINS",-1,"CONTINENT_ASIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][47]={"TERRAIN_PLAINS_HILLS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_GOLD",1},{0,0,0}}
MapToConvert[53][47]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_IRON",1},{0,0,0}}
MapToConvert[52][47]={"TERRAIN_GRASS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][47]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[50][47]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_EUROPE",{{1,2},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[49][47]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_EUROPE",{{1,2},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][47]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{1,1}},{"RESOURCE_IRON",1},{0,0,0}}
MapToConvert[47][47]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[46][47]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{1,3},{1,4}},{"RESOURCE_DEER",1},{0,0,0}}
MapToConvert[45][47]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_AMBER",1},{0,0,0}}
MapToConvert[44][47]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_EUROPE",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][47]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{1,0},{0,-1}},{"RESOURCE_DEER",1},{0,0,0}}
MapToConvert[42][47]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[41][47]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_EUROPE",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][47]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][47]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][47]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][47]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_EUROPE",{{1,2},{0,-1},{0,-1}},{"RESOURCE_CATTLE",1},{0,0,0}}
MapToConvert[36][47]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[35][47]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_STONE",1},{0,0,0}}
MapToConvert[34][47]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][47]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[32][47]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][47]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][47]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][47]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][47]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][47]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[26][47]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][47]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
MapToConvert[24][47]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[23][47]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[22][47]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][47]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][47]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[19][47]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_URANIUM",1},{0,0,0}}
MapToConvert[18][47]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][47]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DEER",1},{0,0,0}}
MapToConvert[16][47]={"TERRAIN_PLAINS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[15][47]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}} -- Lake
MapToConvert[14][47]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][47]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][47]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][47]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{1,1}},{"RESOURCE_DLV_BISON",1},{0,0,0}}
MapToConvert[10][47]={"TERRAIN_PLAINS_MOUNTAIN",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[9][47]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][47]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{1,4}},{"RESOURCE_ALUMINUM",1},{0,0,0}}
MapToConvert[7][47]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,1,0}}
MapToConvert[6][47]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[5][47]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[4][47]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][47]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][47]={"TERRAIN_TUNDRA_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,0}}
MapToConvert[1][47]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,1,0}}
MapToConvert[0][47]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][48]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][48]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{1,0,0}}
MapToConvert[81][48]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{0,-1},{1,1}},{-1,1},{0,1,0}}
MapToConvert[80][48]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COAL",1},{1,0,0}}
MapToConvert[79][48]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[78][48]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][48]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][48]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][48]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{1,0,0}}
MapToConvert[74][48]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[73][48]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_SIBERIA",{{1,2},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][48]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_SIBERIA",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[71][48]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[70][48]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_SIBERIA",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][48]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_SIBERIA",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[68][48]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{0,-1},{1,1}},{"RESOURCE_URANIUM",1},{0,0,0}}
MapToConvert[67][48]={"TERRAIN_TUNDRA_MOUNTAIN",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][48]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,1}} -- Lake
MapToConvert[65][48]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_SIBERIA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][48]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{1,0},{0,-1}},{"RESOURCE_COAL",1},{0,0,0}}
MapToConvert[63][48]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_SIBERIA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][48]={"TERRAIN_PLAINS",-1,"CONTINENT_SIBERIA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][48]={"TERRAIN_PLAINS",-1,"CONTINENT_SIBERIA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][48]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_SIBERIA",{{1,5},{0,-1},{1,4}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[59][48]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_SIBERIA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][48]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][48]={"TERRAIN_PLAINS",-1,"CONTINENT_SIBERIA",{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[56][48]={"TERRAIN_PLAINS",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][48]={"TERRAIN_TUNDRA_MOUNTAIN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][48]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_ALUMINUM",1},{0,0,0}}
MapToConvert[53][48]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[52][48]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[51][48]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[50][48]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_EUROPE",{{0,-1},{1,3},{1,4}},{"RESOURCE_WHEAT",1},{0,0,0}}
MapToConvert[49][48]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_EUROPE",{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[48][48]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_GYPSUM",1},{0,0,0}}
MapToConvert[47][48]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[46][48]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][48]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][48]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[43][48]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{1,1}},{"RESOURCE_AMBER",1},{0,0,0}}
MapToConvert[42][48]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_EUROPE",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][48]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][48]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[39][48]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[38][48]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[37][48]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DEER",1},{0,0,0}}
MapToConvert[36][48]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COAL",1},{0,0,0}}
MapToConvert[35][48]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_IRON",1},{0,0,0}}
MapToConvert[34][48]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[33][48]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][48]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][48]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][48]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][48]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][48]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][48]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][48]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_NITER",1},{0,0,0}}
MapToConvert[25][48]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][48]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
MapToConvert[23][48]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
MapToConvert[22][48]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
MapToConvert[21][48]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][48]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[19][48]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][48]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][48]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][48]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DEER",1},{0,0,0}}
MapToConvert[15][48]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_URANIUM",1},{0,0,0}}
MapToConvert[14][48]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][48]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][48]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[11][48]={"TERRAIN_PLAINS",-1,"CONTINENT_NORTH_AMERICA",{{1,2},{1,0},{1,1}},{"RESOURCE_COAL",1},{0,0,0}}
MapToConvert[10][48]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[9][48]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][48]={"TERRAIN_SNOW_MOUNTAIN",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][48]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[6][48]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][48]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WHALES",1},{0,0,0}}
MapToConvert[4][48]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][48]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DEER",1},{0,1,1}}
MapToConvert[2][48]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
MapToConvert[1][48]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][48]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][49]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][49]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][49]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[80][49]={"TERRAIN_SNOW_MOUNTAIN",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][49]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][49]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[77][49]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][49]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][49]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[74][49]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[73][49]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[72][49]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[71][49]={"TERRAIN_SNOW_MOUNTAIN",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][49]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][49]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{1,0},{0,-1}},{"RESOURCE_FURS",1},{0,0,0}}
MapToConvert[68][49]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_GOLD",1},{0,0,0}}
MapToConvert[67][49]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][49]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[65][49]={"TERRAIN_TUNDRA_MOUNTAIN",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][49]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_SIBERIA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][49]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{1,0},{0,-1}},{"RESOURCE_FURS",1},{0,0,0}}
MapToConvert[62][49]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_SIBERIA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][49]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][49]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][49]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_SIBERIA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][49]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][49]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_SIBERIA",{{1,5},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[56][49]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][49]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][49]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_EUROPE",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[53][49]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{1,4}},{"RESOURCE_FURS",1},{0,0,0}}
MapToConvert[52][49]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][49]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][49]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_EUROPE",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][49]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{1,1}},{"RESOURCE_DYES",1},{0,0,0}}
MapToConvert[48][49]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][49]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[46][49]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][49]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DEER",1},{0,0,0}}
MapToConvert[44][49]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][49]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SALT",1},{0,0,0}}
MapToConvert[42][49]={"TERRAIN_COAST",-1,-1,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][49]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][49]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DEER",1},{0,0,0}}
MapToConvert[39][49]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][49]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][49]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_AMBER",1},{0,0,0}}
MapToConvert[36][49]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][49]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][49]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[33][49]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][49]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][49]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][49]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][49]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][49]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][49]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][49]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][49]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][49]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][49]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][49]={"TERRAIN_TUNDRA","FEATURE_MARSH","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FURS",1},{0,0,0}}
MapToConvert[21][49]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][49]={"TERRAIN_TUNDRA",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[19][49]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][49]={"TERRAIN_TUNDRA",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][49]={"TERRAIN_TUNDRA",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][49]={"TERRAIN_TUNDRA","FEATURE_MARSH","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][49]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][49]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][49]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DEER",1},{0,0,0}}
MapToConvert[12][49]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}} -- Lake
MapToConvert[11][49]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[10][49]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_JADE",1},{0,0,0}}
MapToConvert[9][49]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][49]={"TERRAIN_SNOW_MOUNTAIN",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][49]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][49]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
MapToConvert[5][49]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][49]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[3][49]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
MapToConvert[2][49]={"TERRAIN_TUNDRA","FEATURE_MARSH","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][49]={"TERRAIN_TUNDRA","FEATURE_MARSH","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][49]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[83][50]={"TERRAIN_SNOW_HILLS",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,1}}
MapToConvert[82][50]={"TERRAIN_SNOW_HILLS",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
MapToConvert[81][50]={"TERRAIN_SNOW_HILLS",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
MapToConvert[80][50]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DEER",1},{0,0,0}}
MapToConvert[79][50]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
MapToConvert[78][50]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[77][50]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][50]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][50]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DEER",1},{1,0,1}}
MapToConvert[74][50]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
MapToConvert[73][50]={"TERRAIN_SNOW_MOUNTAIN",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][50]={"TERRAIN_SNOW_MOUNTAIN",-1,"CONTINENT_SIBERIA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][50]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{1,0},{0,-1}},{"RESOURCE_URANIUM",1},{0,0,0}}
MapToConvert[70][50]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_SIBERIA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][50]={"TERRAIN_TUNDRA","FEATURE_MARSH","CONTINENT_SIBERIA",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[68][50]={"TERRAIN_TUNDRA",-1,"CONTINENT_SIBERIA",{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[67][50]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[66][50]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DEER",1},{0,0,0}}
MapToConvert[65][50]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][50]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_SIBERIA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][50]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_SIBERIA",{{1,5},{0,-1},{1,4}},{"RESOURCE_SALT",1},{0,0,0}}
MapToConvert[62][50]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_SIBERIA",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[61][50]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_SIBERIA",{{1,5},{0,-1},{1,4}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[60][50]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_SIBERIA",{{1,5},{0,-1},{1,4}},{"RESOURCE_FURS",1},{0,0,0}}
MapToConvert[59][50]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_SIBERIA",{{1,5},{1,0},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[58][50]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][50]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_EUROPE",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][50]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{1,0},{0,-1}},{"RESOURCE_SILVER",1},{0,0,0}}
MapToConvert[55][50]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][50]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_URANIUM",1},{0,0,0}}
MapToConvert[53][50]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][50]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][50]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][50]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][50]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][50]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DEER",1},{0,0,0}}
MapToConvert[47][50]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_EUROPE",{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[46][50]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][50]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_AMBER",1},{0,0,0}}
MapToConvert[44][50]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[43][50]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_EUROPE",{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[42][50]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[41][50]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][50]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_AMBER",1},{0,0,0}}
MapToConvert[39][50]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[38][50]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][50]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[36][50]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][50]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[34][50]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[33][50]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DEER",1},{0,0,0}}
MapToConvert[32][50]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][50]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][50]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][50]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][50]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[27][50]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_WHALES",1},{0,0,0}}
MapToConvert[26][50]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][50]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][50]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][50]={"TERRAIN_TUNDRA_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{1,2},{0,-1},{1,1}},{"RESOURCE_IRON",1},{0,0,0}}
MapToConvert[22][50]={"TERRAIN_TUNDRA_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[21][50]={"TERRAIN_TUNDRA",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][50]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[19][50]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][50]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][50]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][50]={"TERRAIN_TUNDRA",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][50]={"TERRAIN_TUNDRA",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][50]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][50]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FURS",1},{0,0,0}}
MapToConvert[12][50]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DEER",1},{0,0,0}}
MapToConvert[11][50]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][50]={"TERRAIN_SNOW_MOUNTAIN",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][50]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SILVER",1},{0,0,0}}
MapToConvert[8][50]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][50]={"TERRAIN_SNOW_MOUNTAIN",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][50]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DEER",1},{0,0,0}}
MapToConvert[5][50]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[4][50]={"TERRAIN_SNOW_MOUNTAIN","FEATURE_VOLCANO","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][50]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][50]={"TERRAIN_TUNDRA","FEATURE_MARSH","CONTINENT_NORTH_AMERICA",{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[1][50]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][50]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][51]={"TERRAIN_SNOW_MOUNTAIN",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][51]={"TERRAIN_SNOW_HILLS",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][51]={"TERRAIN_SNOW_HILLS",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_MERCURY",1},{0,0,0}}
MapToConvert[80][51]={"TERRAIN_SNOW_HILLS",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][51]={"TERRAIN_SNOW_MOUNTAIN",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][51]={"TERRAIN_SNOW_MOUNTAIN",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][51]={"TERRAIN_SNOW_MOUNTAIN",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][51]={"TERRAIN_SNOW_HILLS",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][51]={"TERRAIN_SNOW",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[74][51]={"TERRAIN_SNOW",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_MERCURY",1},{0,0,0}}
MapToConvert[73][51]={"TERRAIN_SNOW",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][51]={"TERRAIN_SNOW",-1,"CONTINENT_SIBERIA",{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[71][51]={"TERRAIN_SNOW_HILLS",-1,"CONTINENT_SIBERIA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][51]={"TERRAIN_SNOW_HILLS",-1,"CONTINENT_SIBERIA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][51]={"TERRAIN_TUNDRA_HILLS",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][51]={"TERRAIN_TUNDRA_MOUNTAIN",-1,"CONTINENT_SIBERIA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][51]={"TERRAIN_TUNDRA",-1,"CONTINENT_SIBERIA",{{0,-1},{1,0},{0,-1}},{"RESOURCE_CATTLE",1},{0,0,0}}
MapToConvert[66][51]={"TERRAIN_TUNDRA_HILLS",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][51]={"TERRAIN_TUNDRA_HILLS",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][51]={"TERRAIN_TUNDRA_HILLS",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][51]={"TERRAIN_TUNDRA_HILLS",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][51]={"TERRAIN_TUNDRA_HILLS",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][51]={"TERRAIN_TUNDRA_HILLS",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][51]={"TERRAIN_TUNDRA_HILLS",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][51]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_SIBERIA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][51]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_SIBERIA",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[57][51]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_SIBERIA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][51]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[55][51]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][51]={"TERRAIN_TUNDRA","FEATURE_MARSH","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][51]={"TERRAIN_TUNDRA","FEATURE_MARSH","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][51]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COAL",1},{0,0,0}}
MapToConvert[51][51]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DEER",1},{0,0,0}}
MapToConvert[50][51]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][51]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[48][51]={"TERRAIN_TUNDRA",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][51]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_AMBER",1},{0,0,0}} -- Lake
MapToConvert[46][51]={"TERRAIN_PLAINS","FEATURE_MARSH","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][51]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[44][51]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_EUROPE",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][51]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[42][51]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][51]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][51]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][51]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[38][51]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][51]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[36][51]={"TERRAIN_GRASS","FEATURE_MARSH","CONTINENT_EUROPE",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][51]={"TERRAIN_GRASS_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[34][51]={"TERRAIN_TUNDRA_MOUNTAIN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][51]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][51]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][51]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][51]={"TERRAIN_SNOW_HILLS","FEATURE_EYJAFJALLAJOKULL","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[29][51]={"TERRAIN_SNOW_HILLS","FEATURE_EYJAFJALLAJOKULL","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][51]={"TERRAIN_TUNDRA_HILLS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[27][51]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][51]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][51]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][51]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][51]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][51]={"TERRAIN_SNOW_MOUNTAIN",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][51]={"TERRAIN_TUNDRA_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[20][51]={"TERRAIN_TUNDRA",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[19][51]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[18][51]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][51]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][51]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][51]={"TERRAIN_TUNDRA",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][51]={"TERRAIN_TUNDRA",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][51]={"TERRAIN_TUNDRA",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][51]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][51]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}} -- Lake
MapToConvert[10][51]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][51]={"TERRAIN_SNOW_MOUNTAIN",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][51]={"TERRAIN_SNOW_MOUNTAIN",-1,-1,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][51]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[6][51]={"TERRAIN_SNOW_MOUNTAIN","FEATURE_VOLCANO","CONTINENT_NORTH_AMERICA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[5][51]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SILVER",1},{0,0,0}}
MapToConvert[4][51]={"TERRAIN_SNOW_MOUNTAIN",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[3][51]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][51]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[1][51]={"TERRAIN_SNOW_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][51]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][52]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][52]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][52]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][52]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][52]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][52]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][52]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][52]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][52]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][52]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][52]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][52]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][52]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][52]={"TERRAIN_SNOW",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[69][52]={"TERRAIN_SNOW",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][52]={"TERRAIN_TUNDRA","FEATURE_MARSH","CONTINENT_SIBERIA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][52]={"TERRAIN_TUNDRA_HILLS","FEATURE_MARSH","CONTINENT_SIBERIA",{{0,-1},{1,0},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[66][52]={"TERRAIN_TUNDRA_HILLS",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][52]={"TERRAIN_SNOW_HILLS",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_COPPER",1},{0,0,0}}
MapToConvert[64][52]={"TERRAIN_SNOW_HILLS",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][52]={"TERRAIN_SNOW_HILLS",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_URANIUM",1},{0,0,0}}
MapToConvert[62][52]={"TERRAIN_SNOW_HILLS",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][52]={"TERRAIN_SNOW_HILLS",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][52]={"TERRAIN_TUNDRA",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][52]={"TERRAIN_TUNDRA",-1,"CONTINENT_SIBERIA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][52]={"TERRAIN_TUNDRA","FEATURE_MARSH","CONTINENT_SIBERIA",{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
MapToConvert[57][52]={"TERRAIN_TUNDRA","FEATURE_MARSH","CONTINENT_SIBERIA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][52]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][52]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][52]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][52]={"TERRAIN_TUNDRA",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][52]={"TERRAIN_TUNDRA",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][52]={"TERRAIN_TUNDRA",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SALT",1},{0,0,0}}
MapToConvert[50][52]={"TERRAIN_TUNDRA","FEATURE_MARSH","CONTINENT_EUROPE",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][52]={"TERRAIN_TUNDRA","FEATURE_MARSH","CONTINENT_EUROPE",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][52]={"TERRAIN_TUNDRA",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_NITER",1},{0,0,0}}
MapToConvert[47][52]={"TERRAIN_TUNDRA","FEATURE_MARSH","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[46][52]={"TERRAIN_PLAINS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][52]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[44][52]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
MapToConvert[43][52]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[42][52]={"TERRAIN_GRASS","FEATURE_FOREST","CONTINENT_EUROPE",{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[41][52]={"TERRAIN_TUNDRA_HILLS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][52]={"TERRAIN_TUNDRA_HILLS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][52]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][52]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[37][52]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_FISH",1},{0,0,0}}
MapToConvert[36][52]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_SHEEP",1},{0,0,0}}
MapToConvert[35][52]={"TERRAIN_TUNDRA_HILLS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_STONE",1},{0,0,0}}
MapToConvert[34][52]={"TERRAIN_PLAINS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[33][52]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][52]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][52]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][52]={"TERRAIN_TUNDRA_HILLS","FEATURE_GEOTHERMAL_FISSURE","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[29][52]={"TERRAIN_TUNDRA_HILLS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[28][52]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[27][52]={"TERRAIN_TUNDRA",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[26][52]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_CRABS",1},{0,0,0}}
MapToConvert[25][52]={"TERRAIN_OCEAN","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][52]={"TERRAIN_OCEAN","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][52]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][52]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][52]={"TERRAIN_SNOW_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[20][52]={"TERRAIN_SNOW_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[19][52]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
MapToConvert[18][52]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][52]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][52]={"TERRAIN_SNOW",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][52]={"TERRAIN_SNOW",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_MERCURY",1},{0,0,0}}
MapToConvert[14][52]={"TERRAIN_SNOW",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][52]={"TERRAIN_SNOW",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][52]={"TERRAIN_SNOW_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_DIAMONDS",1},{0,0,0}}
MapToConvert[11][52]={"TERRAIN_SNOW",-1,"CONTINENT_NORTH_AMERICA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[10][52]={"TERRAIN_SNOW",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[9][52]={"TERRAIN_SNOW_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[8][52]={"TERRAIN_SNOW_MOUNTAIN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][52]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{1,5},{1,0},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][52]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
MapToConvert[5][52]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[4][52]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[3][52]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{1,4}},{"RESOURCE_FURS",1},{0,0,0}}
MapToConvert[2][52]={"TERRAIN_SNOW_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_JADE",1},{0,0,0}}
MapToConvert[1][52]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][52]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[83][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[82][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[81][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[80][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[79][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[78][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[77][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[76][53]={"TERRAIN_OCEAN","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[75][53]={"TERRAIN_OCEAN","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[74][53]={"TERRAIN_OCEAN","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[73][53]={"TERRAIN_OCEAN","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[72][53]={"TERRAIN_OCEAN","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[71][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[70][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[69][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[68][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[67][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[66][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[65][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[64][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[63][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[62][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[61][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[60][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[59][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[58][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[57][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[56][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[55][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[54][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[53][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[52][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[51][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[50][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[49][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[48][53]={"TERRAIN_TUNDRA",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[47][53]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[46][53]={"TERRAIN_TUNDRA","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[45][53]={"TERRAIN_COAST",-1,-1,{{0,-1},{1,3},{1,4}},{"RESOURCE_AMBER",1},{0,0,0}}
MapToConvert[44][53]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[43][53]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{1,2},{0,-1},{0,-1}},{"RESOURCE_COAL",1},{0,0,0}}
MapToConvert[42][53]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{1,1}},{"RESOURCE_IRON",1},{0,0,0}}
MapToConvert[41][53]={"TERRAIN_TUNDRA_HILLS","FEATURE_FOREST","CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[40][53]={"TERRAIN_TUNDRA_HILLS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[39][53]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[38][53]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[37][53]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[36][53]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[35][53]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[34][53]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
MapToConvert[33][53]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[32][53]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[31][53]={"TERRAIN_OCEAN",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[30][53]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[29][53]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
MapToConvert[28][53]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
MapToConvert[27][53]={"TERRAIN_SNOW_HILLS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_URANIUM",1},{0,0,0}}
MapToConvert[26][53]={"TERRAIN_TUNDRA_HILLS",-1,"CONTINENT_EUROPE",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[25][53]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[24][53]={"TERRAIN_OCEAN","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[23][53]={"TERRAIN_OCEAN","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[22][53]={"TERRAIN_OCEAN","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[21][53]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
MapToConvert[20][53]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
MapToConvert[19][53]={"TERRAIN_COAST",-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
MapToConvert[18][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[17][53]={"TERRAIN_SNOW",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[16][53]={"TERRAIN_SNOW",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[15][53]={"TERRAIN_SNOW",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[14][53]={"TERRAIN_SNOW",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[13][53]={"TERRAIN_SNOW",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[12][53]={"TERRAIN_SNOW",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[11][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}} -- Lake
MapToConvert[10][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}} -- Lake
MapToConvert[9][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}} -- Lake
MapToConvert[8][53]={"TERRAIN_SNOW_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[7][53]={"TERRAIN_SNOW_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[6][53]={"TERRAIN_SNOW_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
MapToConvert[5][53]={"TERRAIN_SNOW_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[4][53]={"TERRAIN_SNOW_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{"RESOURCE_OIL",1},{0,0,0}}
MapToConvert[3][53]={"TERRAIN_SNOW_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[2][53]={"TERRAIN_SNOW_HILLS",-1,"CONTINENT_NORTH_AMERICA",{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[1][53]={"TERRAIN_COAST","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
MapToConvert[0][53]={"TERRAIN_OCEAN","FEATURE_ICE",-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}

	return MapToConvert
end